@@ -342,7 +342,7 @@ some <b>bold</b> text
 
 	def testCopyPasteParseTreeWithFileLinkInSamePage(self):
 		page = self.notebook.get_page(Path('Test'))
-		page.parse('wiki', '[[./attachment.pdf]]\n[[./attachment.pdf|my attachment]]')
+		page.parse('wiki', '[[./attachment.pdf]]\n[[./attachment.pdf|my attachment]]\n[[file://host/file]]')
 		parsetree = page.get_parsetree()
 		Clipboard.set_parsetree(self.notebook, page, parsetree)
 
@@ -352,12 +352,13 @@ some <b>bold</b> text
 			'<zim-tree><p>'
 			'<link href="./attachment.pdf">./attachment.pdf</link>\n'
 			'<link href="./attachment.pdf">my attachment</link>\n'
+			'<link href="file://host/file">file://host/file</link>\n'
 			'</p></zim-tree>'
-		) # No need to update the link
+		) # No need to update the link - file uri to external host untouched
 
 	def testCopyPasteParseTreeWithFileLinkInDifferentPage(self):
 		page = self.notebook.get_page(Path('Test'))
-		page.parse('wiki', '[[./attachment.pdf]]\n[[./attachment.pdf|my attachment]]')
+		page.parse('wiki', '[[./attachment.pdf]]\n[[./attachment.pdf|my attachment]]\n[[file://host/file]]')
 		parsetree = page.get_parsetree()
 		Clipboard.set_parsetree(self.notebook, page, parsetree)
 
@@ -367,12 +368,13 @@ some <b>bold</b> text
 			'<zim-tree><p>'
 			'<link href="%s">%s</link>\n'
 			'<link href="%s">my attachment</link>\n'
+			'<link href="file://host/file">file://host/file</link>\n'
 			'</p></zim-tree>' % (
 				convert_path_sep('../Test/attachment.pdf'),
 				convert_path_sep('../Test/attachment.pdf'),
 				convert_path_sep('../Test/attachment.pdf')
 			)
-		) # Link updated to point to same target from new location
+		) # Link updated to point to same target from new location - file uri to external host untouched
 
 	def testCopyPasteParseTreeWithFileLinkInDifferentNotebook(self):
 		# NOTE: no proper syntax for this type of link - just abs file link
@@ -413,12 +415,14 @@ some <b>bold</b> text
 		self.notebook = self.setUpNotebook(name='first notebook', content=('Test',), mock=tests.MOCK_ALWAYS_REAL)
 
 		page = self.notebook.get_page(Path('Test'))
-		page.parse('wiki', '{{./attachment.png}}\n{{../OtherPage/otherimage.png}}')
-		page.attachments_folder.file('attachment.png').touch()
+		page.parse('wiki', '{{./attachment1.png}}\n{{attachment2.png}}\n{{../OtherPage/otherimage.png}}\n{{./../OtherPage/otherimage.png}}')
+		page.attachments_folder.file('attachment1.png').touch()
+		page.attachments_folder.file('attachment2.png').touch()
 
 		newpage = self.notebook.get_page(Path('OtherPage'))
-		newfile = newpage.attachments_folder.file('attachment.png')
-		self.assertFalse(newfile.exists())
+		for name in ('attachment1.png', 'attachment2.png'):
+			newfile = newpage.attachments_folder.file(name)
+			self.assertFalse(newfile.exists())
 
 		parsetree = page.get_parsetree()
 		Clipboard.set_parsetree(self.notebook, page, parsetree)
@@ -429,14 +433,20 @@ some <b>bold</b> text
 			'<zim-tree><p>'
 			'<img src="%s" />\n'
 			'<img src="%s" />\n'
+			'<img src="%s" />\n'
+			'<img src="%s" />\n'
 			'</p></zim-tree>' % (
-				convert_path_sep('./attachment.png'),
-				convert_path_sep('./otherimage.png')
+				convert_path_sep('./attachment1.png'),
+				convert_path_sep('./attachment2.png'),
+				convert_path_sep('./otherimage.png'),
+				convert_path_sep('./otherimage.png'),
 			)
 		)
-		# No update on first image, *but* file is copied
-		# Second image is not copied, but src is updates
-		self.assertTrue(newfile.exists())
+		# No update on two attachments, *but* file is copied
+		# External images are not copied, but src is updates
+		for name in ('attachment1.png', 'attachment2.png'):
+			newfile = newpage.attachments_folder.file(name)
+			self.assertTrue(newfile.exists())
 
 	def testCopyPasteParseTreeWithImageInDifferentNotebook(self):
 		self.notebook = self.setUpNotebook(name='first notebook', content=('Test',), mock=tests.MOCK_ALWAYS_REAL)
@@ -574,3 +584,21 @@ some <b>bold</b> text
 			'<link href="wp?Foo">wikipedia Foo</link>\n'
 			'</p></zim-tree>'
 		) # Does not need update - check it is left alone
+
+
+class TestURIData(tests.TestCase):
+
+	def runTest(self):
+		someobject = tests.MockObject()
+		someobject.uri = "file:///foo"
+
+		for (input, uris, text) in (
+			((File("/foo"),), (File("/foo").uri,), File("/foo").path),
+			((File("~/foo"),), (File("~/foo").uri,), "~/foo"),
+			(("file:///foo",), ("file:///foo",), "file:///foo"),
+			((someobject,), ("file:///foo",), "file:///foo"),
+			(("file:///foo", "file:///bar"), ("file:///foo", "file:///bar"), "file:///foo file:///bar"),
+		):
+			data = UriData(*input)
+			self.assertEqual(data.get_data_as(URI_TARGET_ID), uris)
+			self.assertEqual(data.get_data_as(TEXT_TARGET_ID), text)
