@@ -10,7 +10,6 @@ import tests
 
 from zim.formats import *
 from zim.notebook import Path
-from zim.parsing import link_type
 from zim.templates import Template
 
 from xml.etree.ElementTree import ElementTree, Element
@@ -709,6 +708,17 @@ hmmm
 		tree = self.format.Parser().parse(text)
 		self.assertEqual(tree.tostring(), xml)
 
+	def testUnicodeSpecial(self):
+		text = '''
+		1. Some list item\u2029 with stray PARAGRAPH SEPARATOR
+'''
+		xml = '''\
+<?xml version='1.0' encoding='utf-8'?>
+<zim-tree>
+<p><ol indent="2" start="1"><li>Some list item  with stray PARAGRAPH SEPARATOR</li></ol></p></zim-tree>'''
+		tree = self.format.Parser().parse(text)
+		self.assertEqual(tree.tostring(), xml)
+
 
 class TestGFMAutolinks(tests.TestCase):
 	# See https://github.github.com/gfm/#autolinks-extension-
@@ -823,9 +833,9 @@ class TestHtmlFormat(tests.TestCase, TestFormatMixin):
 			template_options={'empty_lines': 'default'}
 		).dump(tree)
 		self.assertEqual(''.join(html),
-			'<h1>head1<a id="head1" class="h_anchor"/></h1>\n\n'
+			'<h1>head1<a id="head1" class="h_anchor"></a></h1>\n\n'
 			'<br>\n\n'
-			'<h2>head2<a id="head2" class="h_anchor"/></h2>\n\n'
+			'<h2>head2<a id="head2" class="h_anchor"></a></h2>\n\n'
 		)
 
 		html = self.format.Dumper(
@@ -833,8 +843,8 @@ class TestHtmlFormat(tests.TestCase, TestFormatMixin):
 			template_options={'empty_lines': 'remove'}
 		).dump(tree)
 		self.assertEqual(''.join(html),
-			'<h1>head1<a id="head1" class="h_anchor"/></h1>\n\n'
-			'<h2>head2<a id="head2" class="h_anchor"/></h2>\n\n'
+			'<h1>head1<a id="head1" class="h_anchor"></a></h1>\n\n'
+			'<h2>head2<a id="head2" class="h_anchor"></a></h2>\n\n'
 		)
 
 		html = self.format.Dumper(
@@ -842,8 +852,8 @@ class TestHtmlFormat(tests.TestCase, TestFormatMixin):
 			template_options={'empty_lines': 'Remove'} # case sensitive
 		).dump(tree)
 		self.assertEqual(''.join(html),
-			'<h1>head1<a id="head1" class="h_anchor"/></h1>\n\n'
-			'<h2>head2<a id="head2" class="h_anchor"/></h2>\n\n'
+			'<h1>head1<a id="head1" class="h_anchor"></a></h1>\n\n'
+			'<h2>head2<a id="head2" class="h_anchor"></a></h2>\n\n'
 		)
 
 
