@@ -16,8 +16,6 @@ import threading
 import time
 
 
-from zim.fs import Dir, File, FS
-
 from zim.main import *
 
 import zim
@@ -118,16 +116,18 @@ class TestGui(tests.TestCase):
 			cmd.run() # Exits without running due to no notebook given in dialog
 
 		### Try again with argument
-		dir = self.create_tmp_dir()
+		folder = self.setUpFolder(mock=tests.MOCK_ALWAYS_REAL)
+		folder.touch()
+
 		cmd = GuiCommand('gui')
-		cmd.parse_options(dir)
+		cmd.parse_options(folder.path)
 		with tests.WindowContext(MainWindow):
 			with tests.LoggingFilter('zim', 'Exception while loading plugin:'):
 				window = cmd.run()
 				self.addCleanup(window.destroy)
 
 		self.assertEqual(window.__class__.__name__, 'MainWindow')
-		self.assertEqual(window.notebook.uri, Dir(dir).uri) # XXX
+		self.assertEqual(window.notebook.uri, folder.uri)
 		self.assertGreaterEqual(len(ConfigManager.preferences['General']['plugins']), 3)
 		self.assertGreaterEqual(len(window.pageview.__zim_extension_objects__), 3)
 
@@ -162,9 +162,10 @@ class TestServer(tests.TestCase):
 		from urllib.request import urlopen
 		from urllib.error import URLError
 
-		dir = self.create_tmp_dir()
+		dir = self.setUpFolder(mock=tests.MOCK_ALWAYS_REAL)
+		dir.touch()
 		cmd = ServerCommand('server')
-		cmd.parse_options(dir)
+		cmd.parse_options(dir.path)
 		t = threading.Thread(target=cmd.run)
 		t.start()
 
