@@ -1,7 +1,7 @@
 
 # Copyright 2009-2017 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
-
+from gi.repository.Gtk import ImageMenuItem
 
 import tests
 
@@ -10,16 +10,14 @@ from zim.plugins import PluginManager
 
 from zim.gui.mainwindow import *
 
-
 is_sensitive = lambda w: w.get_property('sensitive')
 
 
-def setUpMainWindow(notebook, path='Test', plugins=None):
+def setUpMainWindow(notebook, path='Test'):
 	if isinstance(path, str):
 		path = Path(path)
 
 	mainwindow = MainWindow(notebook, page=path)
-	mainwindow.__pluginmanager__ = plugins or PluginManager()
 	mainwindow.init_uistate() # XXX
 	return mainwindow
 
@@ -159,15 +157,27 @@ class TestActions(tests.TestCase):
 		window.open_page_home()
 		self.assertEqual(window.page, Path('Home'))
 
-	def testReloadPage(self):
-		window = setUpMainWindow(self.setUpNotebook(), path='A')
 
-		page1 = window.page
-		window.reload_page()
-		page2 = window.page
-		self.assertEqual(page1, Path('A'))
-		self.assertEqual(page2, Path('A'))
-		self.assertNotEqual(id(page1), id(page2))
+class TestOpenPageImportTextFile(tests.TestCase):
+
+	def runTest(self):
+		# See also uiactions.TestUIActions.testCreateNewPageImportExistingTextFile
+		notebook = self.setUpNotebook(content=('A',))
+		file = notebook.folder.file('B.txt')
+		file.write('Test 123\n') # Not a page, just text !
+
+		window = setUpMainWindow(notebook, path='A')
+
+		def do_import(questiondialog):
+			questiondialog.answer_yes()
+
+		with tests.DialogContext(do_import):
+			window.open_page(Path('B'))
+
+		self.assertEqual(window.page, Path('B'))
+		lines = file.readlines()
+		self.assertEqual(lines[0], 'Content-Type: text/x-zim-wiki\n')
+		self.assertEqual(lines[-1], 'Test 123\n')
 
 
 from gi.repository import Gtk
@@ -191,18 +201,6 @@ class TestTogglingState(tests.TestCase):
 		window.toggle_menubar()
 		#self.assertTrue(window.uistate['show_menubar'])
 
-		self.assertTrue(window.uistate['show_toolbar'])
-		window.toggle_toolbar()
-		self.assertFalse(window.uistate['show_toolbar'])
-		window.toggle_toolbar()
-		self.assertTrue(window.uistate['show_toolbar'])
-
-		self.assertTrue(window.uistate['show_statusbar'])
-		window.toggle_statusbar()
-		self.assertFalse(window.uistate['show_statusbar'])
-		window.toggle_statusbar()
-		self.assertTrue(window.uistate['show_statusbar'])
-
 		mywidget = MockSidePaneWidget()
 		mywidget.show_all()
 		window.add_tab('Test', mywidget, LEFT_PANE)
@@ -213,24 +211,6 @@ class TestTogglingState(tests.TestCase):
 		window.toggle_panes()
 		self.assertTrue(window.uistate['left_pane'][0])
 
-		# note: no default style here - system default unknown
-		for style in (
-			TOOLBAR_ICONS_AND_TEXT,
-			TOOLBAR_ICONS_ONLY,
-			TOOLBAR_TEXT_ONLY,
-		):
-			window.set_toolbar_style(style)
-			self.assertEqual(window.preferences['toolbar_style'], style)
-
-		# note: no default style here - system default unknown
-		for size in (
-			TOOLBAR_ICONS_LARGE,
-			TOOLBAR_ICONS_SMALL,
-			TOOLBAR_ICONS_TINY,
-		):
-			window.set_toolbar_icon_size(size)
-			self.assertEqual(window.preferences['toolbar_size'], size)
-
 		# ..
 		window.toggle_fullscreen()
 		window.toggle_fullscreen()
@@ -246,11 +226,11 @@ class TestSavingPages(tests.TestCase):
 		self.mainwindow.open_page(Path('Non-exsiting:page'))
 		self.assertFalse(self.mainwindow.page.exists())
 		self.assertIsNone(self.mainwindow.page.get_parsetree())
-		self.assertTrue(self.mainwindow.pageview._showing_template) # XXX check HACK
+		self.assertTrue(self.mainwindow.pageview.textview.get_buffer().showing_template)
 		self.mainwindow.pageview.save_page()
 		self.assertTrue(self.mainwindow.page.exists())
 		self.assertIsNotNone(self.mainwindow.page.get_parsetree())
-		self.assertFalse(self.mainwindow.pageview._showing_template) # XXX check HACK
+		self.assertFalse(self.mainwindow.pageview.textview.get_buffer().showing_template)
 
 	def testClose(self):
 		# Specific bug found when trying to close the page while auto-save
@@ -269,3 +249,114 @@ class TestSavingPages(tests.TestCase):
 		self.assertTrue(self.mainwindow.page.modified)
 		self.mainwindow.destroy()
 		self.assertFalse(self.mainwindow.page.modified)
+
+
+class TestMenuDocs(tests.TestCase):
+
+	def setUp(self):
+		self.mainwindow = setUpMainWindow(self.setUpNotebook())
+		self.manual = tests.ZIM_DATA_FOLDER.file('manual/Help/Menu_Items.txt').read()
+
+	def testAllMenuItemsAreDocumented(self):
+		def menuitems(parent, level=0):
+			for mi in parent.get_children():
+				if isinstance(mi, ImageMenuItem):
+					yield mi, level
+				sm = mi.get_submenu()
+				if sm:
+					for mi_sm in menuitems(sm, level+1):
+						yield mi_sm
+
+		for menuitem,level in menuitems(self.mainwindow.menubar):
+			label = menuitem.get_label().replace('_','').rstrip('.')
+			accel_path = menuitem.get_accel_path()
+			accel_label = Gtk.accelerator_get_label(
+				Gtk.AccelMap().lookup_entry(accel_path).key.accel_key,
+				Gtk.AccelMap().lookup_entry(accel_path).key.accel_mods)
+			if accel_label:
+				if '++' in accel_label:
+					label += ' <' + accel_label.replace('++', '><+') + '>'
+				else:
+					label += ' <' + accel_label.replace('+', '><') + '>'
+			if level:
+				label = '**' + label + '**'
+			else:
+				label = '===== ' + label + ' ====='
+			self.assertTrue(label in self.manual, 'Menu item "{}" not documented'.format(label))
+
+
+class TestReadOnlyMainWindow(tests.TestCase):
+
+	def runTest(self):
+		'''Test switching behavior when page opened with read-only status'''
+		notebook = self.setUpNotebook(content=('Test', 'Foo', 'Bar'))
+		window = setUpMainWindow(notebook)
+		self.assertReadWriteState(window)
+
+		notebook.readonly = True # XXX: should never be assigned liek this in apoplication usage
+		notebook._page_cache.clear() # XXX
+		window.open_page(Path('Foo'))
+		self.assertReadOnlyState(window)
+
+		notebook.readonly = False # XXX: should never be assigned liek this in apoplication usage
+		notebook._page_cache.clear() # XXX
+		window.open_page(Path('Bar'))
+		self.assertReadWriteState(window)
+
+	def assertReadWriteState(self, window):
+		for headerbar in (window._headerbar, window._fullscreen_headerbar):
+			title = window.get_title()
+			self.assertNotIn('[' + _('readonly') + ']', title)
+
+		self.assertTrue(window.toggle_editable.get_sensitive())
+		self.assertFalse(window.pageview.readonly)
+
+	def assertReadOnlyState(self, window):
+		for headerbar in (window._headerbar, window._fullscreen_headerbar):
+			title = window.get_title()
+			self.assertIn('[' + _('readonly') + ']', title)
+
+		self.assertFalse(window.toggle_editable.get_sensitive())
+		self.assertTrue(window.pageview.readonly)
+
+
+class TestReadOnlyPageWindow(TestReadOnlyMainWindow):
+
+	def runTest(self):
+		notebook = self.setUpNotebook(content=('Test',))
+		page = notebook.get_page(Path('Test'))
+		window = PageWindow(notebook, page, anchor=None, navigation=None)
+		self.assertReadWriteState(window)
+
+		notebook.readonly = True # XXX: should never be assigned liek this in apoplication usage
+		notebook._page_cache.clear() # XXX
+		page = notebook.get_page(Path('Test'))
+		window = PageWindow(notebook, page, anchor=None, navigation=None)
+		self.assertReadOnlyState(window)
+
+
+class TestUIStateInitSave(tests.TestCase):
+
+	def testMainWindow(self):
+		notebook = self.setUpNotebook(content=('Test',))
+		window = setUpMainWindow(notebook)
+		self.doTest(window)
+
+	def testPageWindow(self):
+		notebook = self.setUpNotebook(content=('Test',))
+		page = notebook.get_page(Path('Test'))
+		window = PageWindow(notebook, page, anchor=None, navigation=None)
+		self.doTest(window)
+
+	def doTest(self, window):
+		from zim.gui.widgets import LEFT_PANE, RIGHT_PANE, TOP_PANE, BOTTOM_PANE
+
+		# Check keys defined in Window.init_uistate()
+		window.show_all()
+		for key in (LEFT_PANE, RIGHT_PANE, TOP_PANE, BOTTOM_PANE):
+			self.assertIn(key, window.uistate)
+
+		# Check Window.save_uistate() is called
+		cb = tests.wrap_method_with_logger(window, 'save_uistate')
+		window.emit('delete-event', None)
+		self.assertTrue(cb.hasBeenCalled)
