@@ -35,19 +35,6 @@ def modify_file_mtime(path, func):
 	#~ print('>>>', m, mtime)
 
 
-#~ class FilterOverWriteWarning(tests.LoggingFilter):
-
-	#~ logger = 'zim.fs'
-	#~ message = 'mtime check failed'
-
-
-#~ class FilterFileMissingWarning(tests.LoggingFilter):
-
-	#~ logger = 'zim.fs'
-	#~ message = 'File missing:'
-
-
-
 def P(path):
 	# Returns a windows path on windows, to make test cases platform
 	# independent and keep them readable
@@ -104,7 +91,11 @@ class TestFilePath(tests.TestCase):
 
 		# Test home folder fallback
 		f = FilePath('~non-existing-user/foo')
-		self.assertEqual(f.path, P('/'.join((HOME.dirname, 'non-existing-user', 'foo'))))
+		homedirname = HOME.dirname or '' # Build environments may have HOME set as e.g. '/builddir'
+		self.assertIn(f.path, (
+			P(os.path.expanduser('~non-existing-user/foo')),
+			P('/'.join((homedirname, 'non-existing-user', 'foo')))
+		))
 
 
 	def testShareDrivePath(self):
@@ -179,7 +170,9 @@ class TestFilePath(tests.TestCase):
 			self.assertEqual(f.get_abspath(p).path, want)
 
 	def testUserpath(self):
-		self.assertTrue(len(HOME.pathnames) >= 2)
+		# Not always true, e.g. a build environment may have home set to
+		# /builddir.
+		#self.assertTrue(len(HOME.pathnames) >= 2)
 
 		f = FilePath('~/foo')
 		self.assertEqual(f.path, HOME.get_childpath('foo').path)
@@ -296,13 +289,22 @@ class TestFS(object):
 		self.assertEqual(file.read(), '')
 		self.assertEqual(file.read_binary(), b'')
 
-		file.write('test 123\n')
-		self.assertEqual(file.read(), 'test 123\n')
-		self.assertEqual(list(file.readlines()), ['test 123\n'])
-		self.assertEqual(list(file), ['test 123\n'])
+		file.write('test 123\ntest abc\n')
+		self.assertEqual(file.read(), 'test 123\ntest abc\n')
+		self.assertEqual(file.read(size=4), 'test')
+		self.assertEqual(file.read(size=100), 'test 123\ntest abc\n')
+		self.assertEqual(file.read(size=-1), 'test 123\ntest abc\n')
+		self.assertEqual(file.read(size=None), 'test 123\ntest abc\n')
+		self.assertEqual(file.readline(), 'test 123\n')
+		self.assertEqual(file.readline(size=4), 'test')
+		self.assertEqual(file.readline(size=100), 'test 123\n')
+		self.assertEqual(file.readline(size=-1), 'test 123\n')
+		self.assertEqual(file.readline(size=None), 'test 123\n')
+		self.assertEqual(list(file.readlines()), ['test 123\n', 'test abc\n'])
+		self.assertEqual(list(file), ['test 123\n', 'test abc\n'])
 
 		file.touch()
-		self.assertEqual(file.read(), 'test 123\n') # no trucation!
+		self.assertEqual(file.read(), 'test 123\ntest abc\n') # no trucation!
 
 		mylines = ['lines1\n', 'lines2\n', 'lines3\n']
 		file.writelines(mylines)
