@@ -1,8 +1,7 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2011-2017 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
-from __future__ import with_statement
+
 
 import tests
 
@@ -125,23 +124,24 @@ Edge case with wrongly nested list
 
 '''
 
+from zim.plugins.tasklist.indexer import TaskParser
+from zim.parsing import parse_date
+
+NO_DATE = '9999'
+
+def t(desc, open=True, start=0, due=NO_DATE, prio=0, tags=''):
+	# Generate a task tuple
+	# 0:open, 1:prio, 2:start, 3:due, 4:tags, 5:desc
+	if tags:
+		tags = set(str(tags).split(','))
+	else:
+		tags = set()
+	return [open, prio, start, due, tags, str(desc)]
+
+
 class TestTaskParser(tests.TestCase):
 
-	def runTest(self):
-		from zim.plugins.tasklist.indexer import TaskParser
-
-		from zim.parsing import parse_date
-		NO_DATE = '9999'
-
-		def t(desc, open=True, start=0, due=NO_DATE, prio=0, tags=''):
-			# Generate a task tuple
-			# 0:open, 1:prio, 2:start, 3:due, 4:tags, 5:desc
-			if tags:
-				tags = set(unicode(tags).split(','))
-			else:
-				tags = set()
-			return [open, prio, start, due, tags, unicode(desc)]
-
+	def testAllCheckboxes(self):
 		mydate = '%04i-%02i-%02i' % parse_date('11/12')
 
 		wanted = [
@@ -230,16 +230,83 @@ class TestTaskParser(tests.TestCase):
 		#~ import pprint; pprint.pprint(tasks)
 		self.assertEqual(tasks, wanted)
 
+	def testLabelledCheckboxes(self):
+		mydate = '%04i-%02i-%02i' % parse_date('11/12')
+
+		wanted = [
+			(t('TODO: test heading with label'), []),
+			(t('A'), []),
+			(t('B'), []),
+			(t('C'), []),
+			(t('FIXME: dus'), []),
+
+			# this time does not inherit due-date from non-task:
+			(t('TODO: BAR !!!', prio=3), []),
+
+			(t('Some more tasks !!!', prio=3, tags='home'), [
+				(t('Foo !', prio=1, tags='home'), []),
+				(t('Bar', prio=3, tags='home'), []),
+			]),
+			(t('TODO: dus'), []),
+			(t('FIXME: jaja - TODO !! @FIXME', prio=2, tags='FIXME'), []),
+			(t('TODO: dus - list item'), []),
+			(t('FIXME: jaja - TODO !! @FIXME - list item', prio=2, tags='FIXME'), []),
+			(t('A', tags='someday'), []),
+			(t('B', tags='someday'), [
+				(t('B-1', tags='someday'), []),
+			]),
+			(t('C', tags='someday'), []),
+			(t('main task', tags='home'), [
+				(t('do this', open=False, tags='home'), []),
+				(t('Next: do that', tags='home'), []),
+				(t('Next: do something else', tags='home'), []),
+			]),
+		]
+
+
+		tree = WikiParser().parse(WIKI_TEXT)
+		tb = TokenBuilder()
+		tree.visit(tb)
+		tokens = tb.tokens
+		testTokenStream(tokens)
+
+		parser = TaskParser(all_checkboxes=False)
+		with tests.LoggingFilter('zim.plugins.tasklist', 'Invalid date format'):
+			tasks = parser.parse(tokens)
+
+		#~ import pprint; pprint.pprint(tasks)
+		self.assertEqual(tasks, wanted)
+
+	def testDate(self):
+		text = '''\
+[ ] Task <2018-12
+[ ] Task >2018-12
+'''
+		wanted = [
+			(t('Task <2018-12', due='2018-12-31'), []),
+			(t('Task >2018-12', start='2018-12-01'), [])
+		]
+
+		tree = WikiParser().parse(text)
+		tb = TokenBuilder()
+		tree.visit(tb)
+		tokens = tb.tokens
+		testTokenStream(tokens)
+
+		parser = TaskParser()
+		tasks = parser.parse(tokens)
+
+		#import pprint; pprint.pprint(tasks)
+		self.assertEqual(tasks, wanted)
+
+
 
 class TestTaskList(tests.TestCase):
 
 	def testIndexing(self):
 		'''Check indexing of tasklist plugin'''
-		klass = PluginManager.get_plugin_class('tasklist')
-		plugin = klass()
-
-		notebook = tests.new_notebook()
-		plugin.extend(notebook)
+		plugin = PluginManager.load_plugin('tasklist')
+		notebook = self.setUpNotebook(content=tests.FULL_NOTEBOOK)
 
 		# Test indexing based on index signals
 		notebook.index.check_and_update()
@@ -255,11 +322,9 @@ class TestTaskList(tests.TestCase):
 				self.assertTrue(not path is None)
 
 	def testTaskListTreeView(self):
-		klass = PluginManager.get_plugin_class('tasklist')
-		plugin = klass()
+		plugin = PluginManager.load_plugin('tasklist')
 
-		notebook = tests.new_notebook()
-		plugin.extend(notebook)
+		notebook = self.setUpNotebook(content=tests.FULL_NOTEBOOK)
 		notebook.index.check_and_update()
 
 		from zim.plugins.tasklist.gui import TaskListTreeView
@@ -275,7 +340,7 @@ class TestTaskList(tests.TestCase):
 
 		# Copy tasklist -> csv
 		from zim.gui.clipboard import Clipboard
-		tests.gtk_activate_menu_item(menu, 'gtk-copy')
+		tests.gtk_activate_menu_item(menu, _('_Copy'))
 		text = Clipboard.get_text()
 		lines = text.splitlines()
 		self.assertTrue(len(lines) > 10)
