@@ -14,13 +14,16 @@ from zim.errors import Error
 from zim.notebook import get_notebook_list, Path, Page, NotebookInfo
 from zim.notebook.notebook import NotebookConfig
 from zim.formats import ParseTree
-from zim.fs import File, Dir
 from zim.gui.clipboard import Clipboard
 
 import zim.gui
 
 
-from zim.gui.uiactions import UIActions, PAGE_EDIT_ACTIONS
+from zim.gui.uiactions import UIActions, PAGE_EDIT_ACTIONS, TrashPageDialog, DeletePageDialog
+
+def strip_file_url_prefix(str):
+	if str.startswith('file://'): return str[7:]
+	return str
 
 
 class EmptyWindowObject(object):
@@ -83,6 +86,25 @@ class TestUIActions(tests.TestCase):
 		with tests.DialogContext(open_new_page):
 			self.uiactions.new_page()
 
+	def testCreateNewPageImportExistingTextFile(self):
+		# See also mainwindow.TestOpenPageImportTextFile
+		file = self.notebook.folder.file('ExistingFile.txt')
+		file.write('Test 123') # Not a page, just text!
+
+		def open_new_page(dialog):
+			dialog.set_input(page='ExistingFile')
+			dialog.assert_response_ok()
+
+		def do_import(questiondialog):
+			questiondialog.answer_yes()
+
+		with tests.DialogContext(open_new_page, do_import):
+			self.uiactions.new_page()
+
+		lines = file.readlines()
+		self.assertEqual(lines[0], 'Content-Type: text/x-zim-wiki\n')
+		self.assertEqual(lines[-1], 'Test 123\n')
+
 	def testCreateNewPageWithRelativePaths(self):
 		self.uiactions.page = self.notebook.get_page(Path('Test:SubPage'))
 
@@ -407,6 +429,22 @@ class TestUIActions(tests.TestCase):
 		with tests.DialogContext(movepage):
 			self.uiactions.move_page(page)
 
+	def testMovePageFailsForExistingTextFile(self):
+		from zim.notebook import PageNotAvailableError
+		file = self.notebook.folder.file('ExistingFile.txt')
+		file.write('Test 123') # Not a page, just text!
+
+		page = self.notebook.get_page(Path('SomeParent:ExistingFile'))
+		page.parse('wiki', 'test 123\n')
+		self.notebook.store_page(page)
+
+		def movepage(dialog):
+			dialog.set_input(parent=':')
+			self.assertRaises(PageNotAvailableError, dialog.do_response_ok)
+
+		with tests.DialogContext(movepage):
+			self.uiactions.move_page(page)
+
 	def testMovePageNonExistingPageFails(self):
 		from zim.notebook import PageNotFoundError
 		page = self.notebook.get_page(Path('NonExistingPage'))
@@ -451,48 +489,40 @@ class TestUIActions(tests.TestCase):
 		self.assertEqual(referrer.dump('wiki'), ['Test [[Test]]\n'])
 
 	def testEditProperties(self):
-		from zim.gui.preferencesdialog import PreferencesDialog
-		from zim.plugins import PluginManager
-
+		from zim.gui.propertiesdialog import PropertiesDialog
 		self.uiactions.widget = Gtk.Window()
-		self.uiactions.widget.__pluginmanager__ = PluginManager()
 
-		def edit_properties(dialog):
-			dialog.set_input(home='NewHome')
-			dialog.assert_response_ok()
-
-		with tests.DialogContext(edit_properties):
+		with tests.DialogContext(
+			(PropertiesDialog, lambda d: d.set_input(home='NewHome', document_root='../my_rel_root'))
+		):
 			self.uiactions.show_properties()
 
 		self.assertEqual(self.notebook.config['Notebook']['home'], Path('NewHome'))
+		self.assertEqual(self.notebook.get_home_page(), Path('NewHome'))
+		self.assertEqual(self.notebook.config['Notebook']['document_root'], '../my_rel_root')
+		from zim.fs import Dir
+		self.assertEqual(self.notebook.document_root, Dir(self.notebook.folder.parent().folder('my_rel_root').path)) # XXX: newfs merge needed
 
 	def testEditPropertiesReadOnly(self):
-		from zim.gui.preferencesdialog import PreferencesDialog
-		from zim.plugins import PluginManager
-
+		from zim.gui.propertiesdialog import PropertiesDialog
 		self.uiactions.widget = Gtk.Window()
-		self.uiactions.widget.__pluginmanager__ = PluginManager()
 
 		self.assertFalse(self.notebook.readonly) # implies attribute exists ..
 		self.notebook.readonly = True
 
-		def edit_properties(dialog):
-			self.assertFalse(dialog.get_input_enabled('home'))
-			dialog.assert_response_ok()
-
-		with tests.DialogContext(edit_properties):
+		with tests.DialogContext(
+			(PropertiesDialog, lambda d: self.assertFalse(d.get_input_enabled('home')))
+		):
 			self.uiactions.show_properties()
 
 	def testPropertiesNotChangedOnCancel(self):
-		from zim.gui.preferencesdialog import PreferencesDialog
-		from zim.plugins import PluginManager
-
+		from zim.gui.propertiesdialog import PropertiesDialog
 		self.uiactions.widget = Gtk.Window()
-		self.uiactions.widget.__pluginmanager__ = PluginManager()
 
 		# In fact this is testig the "cancel" button for all dialogs
 		# which have one ..
 		def edit_properties(dialog):
+			assert isinstance(dialog, PropertiesDialog)
 			dialog.set_input(home='NewHome')
 			dialog.do_response_cancel()
 
@@ -522,7 +552,6 @@ class TestUIActions(tests.TestCase):
 		from zim.plugins import PluginManager
 
 		self.uiactions.widget = Gtk.Window()
-		self.uiactions.widget.__pluginmanager__ = PluginManager()
 
 		with tests.DialogContext(PreferencesDialog):
 			self.uiactions.show_preferences()
@@ -613,15 +642,15 @@ class TestUIActions(tests.TestCase):
 		ZIM_APPLICATION._running = True # HACK
 
 		from zim.gui.server import ServerWindow
-		ServerWindow.show_all = tests.Counter()
-		ServerWindow.present = tests.Counter()
+		ServerWindow.show_all = tests.CallBackLogger()
+		ServerWindow.present = tests.CallBackLogger()
 
 		self.uiactions.show_server_gui()
 
-		self.assertEqual(ServerWindow.present.count, 1)
+		self.assertTrue(ServerWindow.present.hasBeenCalled)
 
 	def testReloadIndex(self):
-		self.uiactions.reload_index()
+		self.uiactions.check_and_update_index()
 
 	def testReloadIndexWhileOngoing(self):
 		from zim.notebook.operations import ongoing_operation
@@ -632,7 +661,7 @@ class TestUIActions(tests.TestCase):
 		next(op_iter)
 		self.assertEqual(ongoing_operation(self.notebook), op)
 
-		self.uiactions.reload_index()
+		self.uiactions.check_and_update_index()
 
 		self.assertIsNone(ongoing_operation(self.notebook))
 
@@ -674,9 +703,9 @@ class TestUIActions(tests.TestCase):
 
 	def testOpenAboutDialog(self):
 		from zim.gui.uiactions import MyAboutDialog
-		MyAboutDialog.run = tests.Counter()
+		MyAboutDialog.run = tests.CallBackLogger()
 		self.uiactions.show_about()
-		self.assertEqual(MyAboutDialog.run.count, 1)
+		self.assertTrue(MyAboutDialog.run.hasBeenCalled)
 
 	def testAccesActionsFromPopupMenu(self):
 		# Test depends on first menu item being "new_page_here"
@@ -727,27 +756,78 @@ class TestUIActionsRealFile(tests.TestCase):
 			self.navigation,
 		)
 
-	def testDeletePageWithTrash(self):
+	def testTrashPage(self):
 		self.assertTrue(self.page.exists())
 
-		with tests.DialogContext(): # fails if dialog shown
+		with tests.DialogContext(TrashPageDialog):
 			self.uiactions.delete_page()
 
 		self.assertFalse(self.page.exists())
 
-	def testDeletePageWithoutTrash(self):
+	def testTrashPageCancel(self):
+		self.assertTrue(self.page.exists())
+
+		def cancel_delete(dialog):
+			assert isinstance(dialog, TrashPageDialog)
+			dialog.do_response_cancel()
+
+		with tests.DialogContext(cancel_delete):
+			self.uiactions.delete_page()
+
+		self.assertTrue(self.page.exists())
+
+	def testTrashPageFallbackToDelete(self):
+		from zim.newfs.helpers import TrashNotSupportedError
+		self.assertTrue(self.page.exists())
+
+		def mock_trash_iter(*a, **k):
+			yield
+			raise TrashNotSupportedError('Test')
+		self.notebook.trash_page_iter = mock_trash_iter
+
+		def fail_trash(dialog):
+			assert isinstance(dialog, TrashPageDialog)
+			dialog.do_response_ok() # NOT assert_repsonse_ok - we want to fail
+
+		def question_yes(dialog):
+			dialog.answer_yes()
+
+		with tests.DialogContext(fail_trash, question_yes, DeletePageDialog):
+			self.uiactions.delete_page()
+
+		self.assertFalse(self.page.exists())
+
+	def testTrashPageFallbackToDeleteCancel(self):
+		from zim.newfs.helpers import TrashNotSupportedError
+		self.assertTrue(self.page.exists())
+
+		def mock_trash_iter(*a, **k):
+			yield
+			raise TrashNotSupportedError('Test')
+		self.notebook.trash_page_iter = mock_trash_iter
+
+		def fail_trash(dialog):
+			assert isinstance(dialog, TrashPageDialog)
+			dialog.do_response_ok() # NOT assert_repsonse_ok - we want to fail
+
+		def question_no(dialog):
+			dialog.answer_no()
+
+		with tests.DialogContext(fail_trash, question_no):
+			self.uiactions.delete_page()
+
+		self.assertTrue(self.page.exists())
+
+	def testDeletePage(self):
 		self.notebook.config['Notebook']['disable_trash'] = True
 		self.assertTrue(self.page.exists())
 
-		def do_delete(dialog):
-			dialog.assert_response_ok()
-
-		with tests.DialogContext(do_delete):
+		with tests.DialogContext(DeletePageDialog):
 			self.uiactions.delete_page()
 
 		self.assertFalse(self.page.exists())
 
-	def testDeletePageWithoutTrashAndChildren(self):
+	def testDeletePageAndChildren(self):
 		self.notebook.config['Notebook']['disable_trash'] = True
 		self.assertTrue(self.page.exists())
 		child = self.notebook.get_page(Path('Test:Child'))
@@ -757,93 +837,83 @@ class TestUIActionsRealFile(tests.TestCase):
 		self.assertTrue(dir.exists())
 		dir.folder('foo').touch()
 
-		def do_delete(dialog):
-			dialog.assert_response_ok()
-
-		with tests.DialogContext(do_delete):
+		with tests.DialogContext(DeletePageDialog):
 			self.uiactions.delete_page()
 
 		self.assertFalse(self.page.exists())
 		self.assertFalse(dir.exists())
 
-	def testDeletePageWithoutTrashCancel(self):
+	def testDeletePageCancel(self):
 		self.notebook.config['Notebook']['disable_trash'] = True
 		self.assertTrue(self.page.exists())
 
-		def do_delete(dialog):
+		def cancel_delete(dialog):
+			assert isinstance(dialog, DeletePageDialog)
 			dialog.do_response_cancel()
 
-		with tests.DialogContext(do_delete):
+		with tests.DialogContext(cancel_delete):
 			self.uiactions.delete_page()
 
 		self.assertTrue(self.page.exists())
 
-	def testDeletePageWithTrashUpdateLinks(self):
-		from zim.config import ConfigManager
-		ConfigManager.preferences['GtkInterface'].input(remove_links_on_delete=True)
+	def testTrashPageUpdateLinks(self):
 		self.assertTrue(self.page.exists())
 
 		referrer = self.notebook.get_page(Path('Referrer'))
 		referrer.parse('wiki', 'Test [[Test]]\n')
 		self.notebook.store_page(referrer)
 
-		with tests.DialogContext(): # fails if dialog shown
-			self.uiactions.delete_page()
+		with tests.DialogContext(
+				(TrashPageDialog, lambda d: d.update_links_checkbutton.set_active(True))
+			):
+				self.uiactions.delete_page()
 
 		self.assertFalse(self.page.exists())
 		self.assertEqual(referrer.dump('wiki'), ['Test Test\n'])
 
-	def testDeletePageWithTrashNoUpdateLinks(self):
-		from zim.config import ConfigManager
-		ConfigManager.preferences['GtkInterface'].input(remove_links_on_delete=False)
+	def testTrashPageNoUpdateLinks(self):
 		self.assertTrue(self.page.exists())
 
 		referrer = self.notebook.get_page(Path('Referrer'))
 		referrer.parse('wiki', 'Test [[Test]]\n')
 		self.notebook.store_page(referrer)
 
-		with tests.DialogContext(): # fails if dialog shown
-			self.uiactions.delete_page()
+		with tests.DialogContext(
+				(TrashPageDialog, lambda d: d.update_links_checkbutton.set_active(False))
+			):
+				self.uiactions.delete_page()
 
 		self.assertFalse(self.page.exists())
 		self.assertEqual(referrer.dump('wiki'), ['Test [[Test]]\n'])
 
-	def testDeletePageWithoutTrashUpdateLinks(self):
-		from zim.config import ConfigManager
-
+	def testDeletePageUpdateLinks(self):
 		self.notebook.config['Notebook']['disable_trash'] = True
-		ConfigManager.preferences['GtkInterface'].input(remove_links_on_delete=True)
 		self.assertTrue(self.page.exists())
 
 		referrer = self.notebook.get_page(Path('Referrer'))
 		referrer.parse('wiki', 'Test [[Test]]\n')
 		self.notebook.store_page(referrer)
 
-		def do_delete(dialog):
-			dialog.assert_response_ok()
-
-		with tests.DialogContext(do_delete):
-			self.uiactions.delete_page()
+		with tests.DialogContext(
+				(DeletePageDialog, lambda d: d.update_links_checkbutton.set_active(True))
+			):
+				self.uiactions.delete_page()
 
 		self.assertFalse(self.page.exists())
 		self.assertEqual(referrer.dump('wiki'), ['Test Test\n'])
 
-	def testDeletePageWithoutTrashNoUpdateLinks(self):
-		from zim.config import ConfigManager
-
+	def testDeletePageNoUpdateLinks(self):
 		self.notebook.config['Notebook']['disable_trash'] = True
-		ConfigManager.preferences['GtkInterface'].input(remove_links_on_delete=False)
 		self.assertTrue(self.page.exists())
 
 		referrer = self.notebook.get_page(Path('Referrer'))
 		referrer.parse('wiki', 'Test [[Test]]\n')
 		self.notebook.store_page(referrer)
 
-		def do_delete(dialog):
-			dialog.assert_response_ok()
-
-		with tests.DialogContext(do_delete):
-			self.uiactions.delete_page()
+		with tests.DialogContext(
+				(DeletePageDialog, lambda d: d.update_links_checkbutton.set_active(False))
+			):
+				self.uiactions.delete_page()
 
 		self.assertFalse(self.page.exists())
 		self.assertEqual(referrer.dump('wiki'), ['Test [[Test]]\n'])
@@ -853,7 +923,7 @@ class TestUIActionsRealFile(tests.TestCase):
 		folder.touch()
 
 		def open_folder(cmd):
-			self.assertEqual(cmd[-1], folder.path)
+			self.assertEqual(strip_file_url_prefix(cmd[-1]), folder.path)
 
 		with tests.ApplicationContext(open_folder):
 			with tests.DialogContext():
@@ -867,7 +937,7 @@ class TestUIActionsRealFile(tests.TestCase):
 			dialog.answer_yes()
 
 		def open_folder(cmd):
-			self.assertEqual(cmd[-1], folder.path)
+			self.assertEqual(strip_file_url_prefix(cmd[-1]), folder.path)
 
 		with tests.ApplicationContext(open_folder):
 			with tests.DialogContext(create_folder):
@@ -875,7 +945,7 @@ class TestUIActionsRealFile(tests.TestCase):
 
 	def testOpenNotebookFolder(self):
 		def open_folder(cmd):
-			self.assertEqual(cmd[-1], self.notebook.folder.path)
+			self.assertEqual(strip_file_url_prefix(cmd[-1]), self.notebook.folder.path)
 
 		with tests.ApplicationContext(open_folder):
 			self.uiactions.open_notebook_folder()
@@ -886,7 +956,7 @@ class TestUIActionsRealFile(tests.TestCase):
 		self.notebook.document_root.touch()
 
 		def open_folder(cmd):
-			self.assertEqual(cmd[-1], self.notebook.document_root.path)
+			self.assertEqual(strip_file_url_prefix(cmd[-1]), self.notebook.document_root.path)
 
 		with tests.ApplicationContext(open_folder):
 			with tests.DialogContext():
@@ -924,5 +994,21 @@ class TestUIActionsRealFile(tests.TestCase):
 				self.uiactions.edit_page_source()
 
 		newtext = self.page.dump('plain')
-		self.assertEqual(signals['page-changed'], [(True,)]) # boolean for external change
+		self.assertEqual(signals['storage-changed'], [(True,)]) # boolean for external change
 		self.assertNotEqual(oldtext, newtext)
+
+
+class TestManualProperties(tests.TestCase):
+
+	def runTest(self):
+		from zim.gui.propertiesdialog import notebook_properties
+
+		with open('./data/manual/Help/Properties.txt') as fh:
+			manual = fh.read()
+
+		for pref in notebook_properties:
+			label = pref[2]
+			if '\n' in label:
+				label, x = label.split('\n', 1)
+				label = label.rstrip(',')
+			self.assertTrue(label in manual, 'Property "%s" not documented in manual page' % label)
