@@ -54,6 +54,13 @@ zim ($DEBNEW) unstable; urgency=medium
 EOF
 
 
+# Update AppData
+
+echo "Updating xdg/org.zim_wiki.Zim.appdata.xml"
+
+./tools/add_release_to_appdata.py
+
+
 # Build package & test it
 
 echo "Building ..."
