@@ -1,31 +1,31 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
 '''Interactively dumps clipboard contents to stdout'''
 
-import gtk
+from gi.repository import Gtk
 import sys
 
-clipboard = gtk.Clipboard()
+clipboard = Gtk.Clipboard()
 targets = clipboard.wait_for_targets()
 
-print "="*80
-print "Enter a number to see a specific target, or <Enter> to exit"
-print "Available targets:"
+print("=" * 80)
+print("Enter a number to see a specific target, or <Enter> to exit")
+print("Available targets:")
 for i in range(len(targets)):
-	print i, targets[i]
+	print(i, targets[i])
 
 line = sys.stdin.readline().strip()
 while line:
 	target = targets[int(line)]
-	print '>>>>', target
+	print('>>>>', target)
 	selection = clipboard.wait_for_contents(target)
 	if selection:
 		text = selection.get_text()
 		if not text is None:
-			print '== Text:', text
+			print('== Text:', text)
 		else:
-			print '== Data:', selection.data
+			print('== Data:', selection.data)
 	else:
-		print '== No contents'
-	print '<<<<'
+		print('== No contents')
+	print('<<<<')
 	line = sys.stdin.readline().strip()
