@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2012 Yao-Po Wang <blue119@gmail.com>
 # Copyright 2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>
@@ -25,21 +24,23 @@ info = {
 class Dumper(TextDumper):
 
 	BULLETS = {
-		UNCHECKED_BOX:	u'- \u2610',
-		XCHECKED_BOX:	u'- \u2612',
-		CHECKED_BOX:	u'- \u2611',
-		BULLET:		u'-',
+		UNCHECKED_BOX: '- \u2610',
+		XCHECKED_BOX: '- \u2612',
+		CHECKED_BOX: '- \u2611',
+		MIGRATED_BOX: '- \u25B7',
+		TRANSMIGRATED_BOX: '- \u25C1',
+		BULLET: '-',
 	}
 
 	TAGS = {
-		EMPHASIS:	('*', '*'),
-		STRONG:		('**', '**'),
-		MARK:		('', ''), # TODO, no directly way to do this in rst
-		STRIKE:		('', ''), # TODO, no directly way to do this in rst
-		VERBATIM:	("``", "``"),
-		TAG:		('', ''), # No additional annotation (apart from the visible @)
-		SUBSCRIPT:	('\\ :sub:`', '`\\ '),
-		SUPERSCRIPT:	('\\ :sup:`', '`\\ '),
+		EMPHASIS: ('*', '*'),
+		STRONG: ('**', '**'),
+		MARK: ('', ''), # TODO, no directly way to do this in rst
+		STRIKE: ('', ''), # TODO, no directly way to do this in rst
+		VERBATIM: ("``", "``"),
+		TAG: ('', ''), # No additional annotation (apart from the visible @)
+		SUBSCRIPT: ('\\ :sub:`', '`\\ '),
+		SUPERSCRIPT: ('\\ :sup:`', '`\\ '),
 	}
 	# TODO tags other than :sub: and :sup: may also need surrounding whitespace, deal with this in post process (join) action ?
 	# IDEM for blocks like images and objects, how to enforce empty lines and how to deal with inline images..
@@ -53,10 +54,12 @@ class Dumper(TextDumper):
 	def dump_h(self, tag, attrib, strings):
 		# Underlined headings
 		level = int(attrib['level'])
-		if level < 1:   level = 1
-		elif level > 4: level = 4
-		char = self.HEADING_UNDERLINE[level-1]
-		heading = u''.join(strings)
+		if level < 1:
+			level = 1
+		elif level > 4:
+			level = 4
+		char = self.HEADING_UNDERLINE[level - 1]
+		heading = ''.join(strings)
 		underline = char * len(heading)
 		return [heading + '\n', underline]
 
@@ -72,22 +75,21 @@ class Dumper(TextDumper):
 		assert 'href' in attrib, \
 			'BUG: link misses href: %s "%s"' % (attrib, strings)
 		href = self.linker.link(attrib['href'])
-		text = u''.join(strings) or href
+		text = ''.join(strings) or href
 		return '`%s <%s>`_' % (text, href)
 
 	def dump_img(self, tag, attrib, strings=None):
 		src = self.linker.img(attrib['src'])
 		text = '.. image:: %s\n' % src
 
-		items = attrib.items()
-		items.sort() # unit tests don't like random output
+		items = sorted(attrib.items())
 		for k, v in items:
 			if k == 'src' or k.startswith('_'):
 				continue
 			elif v: # skip None, "" and 0
 				text += '   :%s: %s\n' % (k, v)
 
-		return text + '\n'
+		return [text + '\n']
 
 		# TODO use text for caption (with full recursion)
 		# can be done using "figure" directive
@@ -111,18 +113,7 @@ class Dumper(TextDumper):
 			table += [rowline(line) for line in row]
 			table.append(rowsep('-'))
 
-		return map(lambda line: line+"\n", table)
+		return [line + "\n" for line in table]
 
-	def dump_thead(self, tag, attrib, strings):
-		return [strings]
-
-	def dump_th(self, tag, attrib, strings):
-		strings = [s.replace('|', '∣') for s in strings]
-		return strings
-
-	def dump_trow(self, tag, attrib, strings):
-		return [strings]
-
-	def dump_td(self, tag, attrib, strings):
-		strings = [s.replace('|', '∣') for s in strings]
-		return strings
+	def dump_line(self, tag, attrib, strings=None):
+		return '\n{}\n'.format('*' * 5)
