@@ -1,27 +1,26 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2010-2014 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
-import gtk
+from gi.repository import Gtk
 
 import re
 from datetime import date as dateclass
 
-from zim.fs import Dir
+from zim.fs import Dir, isabs
 
-from zim.plugins import PluginClass, WindowExtension, extends
-from zim.main import GtkCommand
+from zim.plugins import PluginClass
 from zim.actions import action
 from zim.config import data_file, ConfigManager
 from zim.notebook import Path, Notebook, NotebookInfo, \
 	resolve_notebook, build_notebook
+from zim.templates import get_template
+from zim.main import GtkCommand, ZIM_APPLICATION
+
+from zim.gui.mainwindow import MainWindowExtension
 from zim.gui.widgets import Dialog, ScrolledTextView, IconButton, \
 	InputForm, QuestionDialog
 from zim.gui.clipboard import Clipboard, SelectionClipboard
 from zim.gui.notebookdialog import NotebookComboBox
-from zim.templates import get_template
-
-from zim.main import ZIM_APPLICATION
 
 
 import logging
@@ -95,6 +94,12 @@ class QuickNotePluginCommand(GtkCommand):
 			self.opts['append'] = \
 				self.opts['append'].lower() == 'true'
 
+		if self.opts.get('attachments', None):
+			if isabs(self.opts['attachments']):
+				self.opts['attachments'] = Dir(self.opts['attachments'])
+			else:
+				self.opts['attachments'] = Dir((self.pwd, self.opts['attachments']))
+
 	def get_text(self):
 		if 'input' in self.opts:
 			if self.opts['input'] == 'stdin':
@@ -105,45 +110,56 @@ class QuickNotePluginCommand(GtkCommand):
 					SelectionClipboard.get_text() \
 					or Clipboard.get_text()
 			else:
-				raise AssertionError, 'Unknown input type: %s' % self.opts['input']
+				raise AssertionError('Unknown input type: %s' % self.opts['input'])
 		else:
-			text = self.opts.get('text')
+			text = self.opts.get('text', '')
 
 		if text and 'encoding' in self.opts:
 			if self.opts['encoding'] == 'base64':
 				import base64
-				text = base64.b64decode(text)
+				text = base64.b64decode(text).decode('UTF-8')
 			elif self.opts['encoding'] == 'url':
 				from zim.parsing import url_decode, URL_ENCODE_DATA
 				text = url_decode(text, mode=URL_ENCODE_DATA)
 			else:
-				raise AssertionError, 'Unknown encoding: %s' % self.opts['encoding']
-
-		if text and not isinstance(text, unicode):
-			text = text.decode('utf-8')
+				raise AssertionError('Unknown encoding: %s' % self.opts['encoding'])
 
+		assert isinstance(text, str), '%r is not decoded' % text
 		return text
 
-	def run(self):
+	def run_local(self):
+		# Try to run dialog from local process
+		# - prevents issues where dialog pop behind other applications
+		#   (desktop preventing new window of existing process to hijack focus)
+		# - e.g. capturing stdin requires local process
 		if self.opts.get('help'):
-			print usagehelp # TODO handle this in the base class
+			print(usagehelp) # TODO handle this in the base class
 		else:
-			if 'notebook' in self.opts:
-				notebook = resolve_notebook(self.opts['notebook'])
-			else:
-				notebook = None
+			dialog = self.build_dialog()
+			dialog.run()
+		return True # Done - Don't call run() as well
 
-			dialog = QuickNoteDialog(None,
-				notebook=notebook,
-				namespace=self.opts.get('namespace'),
-				basename=self.opts.get('basename'),
-				append=self.opts.get('append'),
-				text=self.get_text(),
-				template_options=self.template_options,
-				attachments=self.opts.get('attachments')
-			)
-			dialog.show_all()
-			return dialog
+	def run(self):
+		# If called from primary process just run the dialog
+		return self.build_dialog()
+
+	def build_dialog(self):
+		if 'notebook' in self.opts:
+			notebook = resolve_notebook(self.opts['notebook'])
+		else:
+			notebook = None
+
+		dialog = QuickNoteDialog(None,
+			notebook=notebook,
+			namespace=self.opts.get('namespace'),
+			basename=self.opts.get('basename'),
+			append=self.opts.get('append'),
+			text=self.get_text(),
+			template_options=self.template_options,
+			attachments=self.opts.get('attachments')
+		)
+		dialog.show_all()
+		return dialog
 
 
 class QuickNotePlugin(PluginClass):
@@ -165,49 +181,57 @@ This is a core plugin shipping with zim.
 	#~ )
 
 
-@extends('MainWindow')
-class MainWindowExtension(WindowExtension):
+class QuickNoteMainWindowExtension(MainWindowExtension):
 
-	uimanager_xml = '''
-	<ui>
-		<menubar name='menubar'>
-			<menu action='file_menu'>
-				<placeholder name="open_items">
-					<menuitem action="show_quick_note" />
-				</placeholder>
-			</menu>
-		</menubar>
-	</ui>
-	'''
-
-	@action(_('Quick Note...'), stock='gtk-new') # T: menu item
+	@action(_('Quick Note...'), menuhints='notebook') # T: menu item
 	def show_quick_note(self):
-		ui = self.window.ui # XXX
-		notebook = self.window.ui.notebook # XXX
-		dialog = BoundQuickNoteDialog.unique(self, self.window, notebook, ui)
+		dialog = QuickNoteDialog.unique(self, self.window, self.window.notebook)
 		dialog.show()
 
 
-class BoundQuickNoteDialog(Dialog):
+class QuickNoteDialog(Dialog):
 	'''Dialog bound to a specific notebook'''
 
-	def __init__(self, window, notebook, ui,
+	def __init__(self, window, notebook=None,
 		page=None, namespace=None, basename=None,
 		append=None, text=None, template_options=None, attachments=None
 	):
+		assert page is None, 'TODO'
+
+		self.config = ConfigManager.get_config_dict('quicknote.conf')
+		self.uistate = self.config['QuickNoteDialog']
+
 		Dialog.__init__(self, window, _('Quick Note'))
-		self._ui = ui
 		self._updating_title = False
 		self._title_set_manually = not basename is None
 		self.attachments = attachments
 
-		self.uistate.setdefault('namespace', None, basestring)
-		namespace = namespace or self.uistate['namespace']
+		if notebook and not isinstance(notebook, str):
+			notebook = notebook.uri
+
+		self.uistate.setdefault('lastnotebook', None, str)
+		if self.uistate['lastnotebook']:
+			notebook = notebook or self.uistate['lastnotebook']
+			self.config['Namespaces'].setdefault(notebook, None, str)
+			namespace = namespace or self.config['Namespaces'][notebook]
+
+		self.form = InputForm()
+		self.vbox.pack_start(self.form, False, True, 0)
+
+		# TODO dropdown could use an option "Other..."
+		label = Gtk.Label(label=_('Notebook') + ': ')
+		label.set_alignment(0.0, 0.5)
+		self.form.attach(label, 0, 1, 0, 1, xoptions=Gtk.AttachOptions.FILL)
+			# T: Field to select Notebook from drop down list
+		self.notebookcombobox = NotebookComboBox(current=notebook)
+		self.notebookcombobox.connect('changed', self.on_notebook_changed)
+		self.form.attach(self.notebookcombobox, 1, 2, 0, 1)
 
-		self.form = InputForm(notebook=notebook)
-		self.vbox.pack_start(self.form, False)
 		self._init_inputs(namespace, basename, append, text, template_options)
 
+		self.uistate['lastnotebook'] = notebook
+		self._set_autocomplete(notebook)
+
 	def _init_inputs(self, namespace, basename, append, text, template_options, custom=None):
 		if template_options is None:
 			template_options = {}
@@ -219,18 +243,18 @@ class BoundQuickNoteDialog(Dialog):
 		else:
 			page = namespace or basename
 
-		self.form.add_inputs( (
+		self.form.add_inputs((
 				('page', 'page', _('Page')),
 				('namespace', 'namespace', _('Page section')), # T: text entry field
 				('new_page', 'bool', _('Create a new page for each note')), # T: checkbox in Quick Note dialog
 				('basename', 'string', _('Title')) # T: text entry field
-			) )
+			))
 		self.form.update({
 				'page': page,
 				'namespace': namespace,
 				'new_page': True,
 				'basename': basename,
-			} )
+			})
 
 		self.uistate.setdefault('open_page', True)
 		self.uistate.setdefault('new_page', True)
@@ -260,10 +284,10 @@ class BoundQuickNoteDialog(Dialog):
 		switch_input()
 		self.form.widgets['new_page'].connect('toggled', switch_input)
 
-		self.open_page_check = gtk.CheckButton(_('Open _Page')) # T: Option in quicknote dialog
+		self.open_page_check = Gtk.CheckButton.new_with_mnemonic(_('Open _Page')) # T: Option in quicknote dialog
 			# Don't use "O" as accelerator here to avoid conflict with "Ok"
 		self.open_page_check.set_active(self.uistate['open_page'])
-		self.action_area.pack_start(self.open_page_check, False)
+		self.action_area.pack_start(self.open_page_check, False, True, 0)
 		self.action_area.set_child_secondary(self.open_page_check, True)
 
 		# Add the main textview and hook up the basename field to
@@ -271,7 +295,7 @@ class BoundQuickNoteDialog(Dialog):
 		window, textview = ScrolledTextView()
 		self.textview = textview
 		self.textview.set_editable(True)
-		self.vbox.add(window)
+		self.vbox.pack_start(window, True, True, 0)
 
 		self.form.widgets['basename'].connect('changed', self.on_title_changed)
 		self.textview.get_buffer().connect('changed', self.on_text_changed)
@@ -292,8 +316,37 @@ class BoundQuickNoteDialog(Dialog):
 
 		self.connect('delete-event', self.do_delete_event)
 
+	def on_notebook_changed(self, o):
+		notebook = self.notebookcombobox.get_notebook()
+		if not notebook or notebook == self.uistate['lastnotebook']:
+			return
+
+		self.uistate['lastnotebook'] = notebook
+		self.config['Namespaces'].setdefault(notebook, None, str)
+		namespace = self.config['Namespaces'][notebook]
+		if namespace:
+			self.form['namespace'] = namespace
+
+		self._set_autocomplete(notebook)
+
+	def _set_autocomplete(self, notebook):
+		if notebook:
+			try:
+				if isinstance(notebook, str):
+					notebook = NotebookInfo(notebook)
+				obj, x = build_notebook(notebook)
+				self.form.widgets['namespace'].notebook = obj
+				self.form.widgets['page'].notebook = obj
+				logger.debug('Notebook for autocomplete: %s (%s)', obj, notebook)
+			except:
+				logger.exception('Could not set notebook: %s', notebook)
+		else:
+			self.form.widgets['namespace'].notebook = None
+			self.form.widgets['page'].notebook = None
+			logger.debug('Notebook for autocomplete unset')
+
 	def do_response(self, id):
-		if id == gtk.RESPONSE_DELETE_EVENT:
+		if id == Gtk.ResponseType.DELETE_EVENT:
 			if self.textview.get_buffer().get_modified():
 				ok = QuestionDialog(self, _('Discard note?')).run()
 					# T: confirm closing quick note dialog
@@ -318,12 +371,16 @@ class BoundQuickNoteDialog(Dialog):
 		Dialog.show(self)
 
 	def save_uistate(self):
+		notebook = self.notebookcombobox.get_notebook()
+		self.uistate['lastnotebook'] = notebook
 		self.uistate['new_page'] = self.form['new_page']
 		self.uistate['open_page'] = self.open_page_check.get_active()
-		if self.uistate['new_page']:
-			self.uistate['namespace'] = self.form['namespace']
-		else:
-			self.uistate['namespace'] = self.form['page']
+		if notebook is not None:
+			if self.uistate['new_page']:
+				self.config['Namespaces'][notebook] = self.form['namespace']
+			else:
+				self.config['Namespaces'][notebook] = self.form['page']
+		self.config.write()
 
 	def on_title_changed(self, o):
 		o.set_input_valid(True)
@@ -334,8 +391,8 @@ class BoundQuickNoteDialog(Dialog):
 		if not self._title_set_manually:
 			# Automatically generate a (valid) page name
 			self._updating_title = True
-			bounds = buffer.get_bounds()
-			title = buffer.get_text(*bounds).strip()[:50]
+			start, end = buffer.get_bounds()
+			title = start.get_text(end).strip()[:50]
 				# Cut off at 50 characters to prevent using a whole paragraph
 			title = title.replace(':', '')
 			if '\n' in title:
@@ -349,8 +406,8 @@ class BoundQuickNoteDialog(Dialog):
 
 	def do_response_ok(self):
 		buffer = self.textview.get_buffer()
-		bounds = buffer.get_bounds()
-		text = buffer.get_text(*bounds)
+		start, end = buffer.get_bounds()
+		text = start.get_text(end)
 
 		# HACK: change "[]" at start of line into "[ ]" so checkboxes get inserted correctly
 		text = re.sub(r'(?m)^(\s*)\[\](\s)', r'\1[ ]\2', text)
@@ -377,20 +434,21 @@ class BoundQuickNoteDialog(Dialog):
 				return False
 
 			path = self.form['page']
-			self.append_to_page(notebook, path, '\n------\n'+text)
+			self.append_to_page(notebook, path, '\n------\n' + text)
 
 		if self.attachments:
-			self.import_attachments(notebook, path)
+			self.import_attachments(notebook, path, self.attachments)
 
 		if self.open_page_check.get_active():
 			self.hide()
-			self.open_page(notebook, path)
+			ZIM_APPLICATION.present(notebook, path)
 
 		return True
 
 	def _get_notebook(self):
-		# Overloaded below
-		return self._ui.notebook
+		uri = self.notebookcombobox.get_notebook()
+		notebook, p = build_notebook(Dir(uri))
+		return notebook
 
 	def create_new_page(self, notebook, path, text):
 		page = notebook.get_new_page(path)
@@ -402,104 +460,11 @@ class BoundQuickNoteDialog(Dialog):
 		page.parse('wiki', text, append=True) # FIXME format hard coded
 		notebook.store_page(page)
 
-	def import_attachments(self, notebook, path, folder):
+	def import_attachments(self, notebook, path, dir):
 		attachments = notebook.get_attachments_dir(path)
-		for name in folder.list():
+		attachments = Dir(attachments.path) # XXX
+		for name in dir.list():
 			# FIXME could use list objects, or list_files()
 			file = dir.file(name)
 			if not file.isdir():
 				file.copyto(attachments)
-
-	def open_page(self, notebook, path):
-		assert notebook == self._ui.notebook
-		self._ui.present(path)
-
-
-class QuickNoteDialog(BoundQuickNoteDialog):
-	'''Dialog which includes a notebook chooser'''
-
-	def __init__(self, window, notebook=None,
-		page=None, namespace=None, basename=None,
-		append=None, text=None, template_options=None, attachments=None
-	):
-		assert page is None, 'TODO'
-		manager = ConfigManager() # FIXME should be passed in
-		self.config = manager.get_config_dict('quicknote.conf')
-		self.uistate = self.config['QuickNoteDialog']
-
-		Dialog.__init__(self, window, _('Quick Note'))
-		self._updating_title = False
-		self._title_set_manually = not basename is None
-		self.attachments = attachments
-
-		if notebook and not isinstance(notebook, basestring):
-			notebook = notebook.uri
-
-		self.uistate.setdefault('lastnotebook', None, basestring)
-		if self.uistate['lastnotebook']:
-			notebook = notebook or self.uistate['lastnotebook']
-			self.config['Namespaces'].setdefault(notebook, None, basestring)
-			namespace = namespace or self.config['Namespaces'][notebook]
-
-		self.form = InputForm()
-		self.vbox.pack_start(self.form, False)
-
-		# TODO dropdown could use an option "Other..."
-		label = gtk.Label(_('Notebook')+': ')
-		label.set_alignment(0.0, 0.5)
-		self.form.attach(label, 0,1, 0,1, xoptions=gtk.FILL)
-			# T: Field to select Notebook from drop down list
-		self.notebookcombobox = NotebookComboBox(current=notebook)
-		self.notebookcombobox.connect('changed', self.on_notebook_changed)
-		self.form.attach(self.notebookcombobox, 1,2, 0,1)
-
-		self._init_inputs(namespace, basename, append, text, template_options)
-
-		self.uistate['lastnotebook'] = notebook
-		self._set_autocomplete(notebook)
-
-	def save_uistate(self):
-		notebook = self.notebookcombobox.get_notebook()
-		self.uistate['lastnotebook'] = notebook
-		self.uistate['new_page'] = self.form['new_page']
-		self.uistate['open_page'] = self.open_page_check.get_active()
-		if notebook is not None:
-			if self.uistate['new_page']:
-				self.config['Namespaces'][notebook] = self.form['namespace']
-			else:
-				self.config['Namespaces'][notebook] = self.form['page']
-		self.config.write()
-
-	def on_notebook_changed(self, o):
-		notebook = self.notebookcombobox.get_notebook()
-		if not notebook or notebook == self.uistate['lastnotebook']:
-			return
-
-		self.uistate['lastnotebook'] = notebook
-		self.config['Namespaces'].setdefault(notebook, None, basestring)
-		namespace = self.config['Namespaces'][notebook]
-		if namespace:
-			self.form['namespace'] = namespace
-
-		self._set_autocomplete(notebook)
-
-	def _set_autocomplete(self, notebook):
-		if notebook:
-			if isinstance(notebook, basestring):
-				notebook = NotebookInfo(notebook)
-			obj, x = build_notebook(notebook)
-			self.form.widgets['namespace'].notebook = obj
-			self.form.widgets['page'].notebook = obj
-			logger.debug('Notebook for autocomplete: %s (%s)', obj, notebook)
-		else:
-			self.form.widgets['namespace'].notebook = None
-			self.form.widgets['page'].notebook = None
-			logger.debug('Notebook for autocomplete unset')
-
-	def _get_notebook(self):
-		uri = self.notebookcombobox.get_notebook()
-		notebook, p = build_notebook(Dir(uri))
-		return notebook
-
-	def open_page(self, notebook, path):
-		ZIM_APPLICATION.run('--gui', notebook.uri, path.name)
