@@ -1,28 +1,36 @@
-# -*- coding: utf-8 -*-
 
-# Copyright 2009-2014 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2009-2018 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 # Copyright 2012 Damien Accorsi <damien.accorsi@free.fr>
 
-from __future__ import with_statement
 
-import gobject
-import gtk
+
+from gi.repository import GObject
+from gi.repository import Gtk
 
 import os
 import logging
+import threading
 
-from zim.fs import FS, File, TmpFile
-from zim.plugins import PluginClass, extends, WindowExtension, ObjectExtension
+from functools import partial
+
+from zim.fs import TmpFile
+from zim.newfs import LocalFolder
+from zim.plugins import PluginClass, find_extension
 from zim.actions import action
 from zim.signals import ConnectorMixin
 from zim.errors import Error
 from zim.applications import Application
 from zim.gui.applications import DesktopEntryFile
 from zim.config import value_is_coord, data_dirs
+from zim.notebook import NotebookExtension
+from zim.notebook.operations import NotebookState, SimpleAsyncOperation
+from zim.utils import natural_sort_key
+
+from zim.gui.mainwindow import MainWindowExtension
 from zim.gui.widgets import ErrorDialog, QuestionDialog, Dialog, \
 	PageEntry, IconButton, SingleClickTreeView, \
-	ScrolledWindow, ScrolledTextView, VPaned
-from zim.utils import natural_sort_key, FunctionThread
+	ScrolledWindow, ScrolledTextView, VPaned, \
+	ProgressDialog
 
 
 if os.environ.get('ZIM_TEST_RUNNING'):
@@ -57,155 +65,153 @@ This is a core plugin shipping with zim.
 	}
 
 	plugin_preferences = (
-		('autosave', 'bool', _('Autosave version on regular intervals'), False), # T: Label for plugin preference
+		('autosave', 'bool', _('Autosave version when the notebook is closed'), False), # T: Label for plugin preference
+		('autosave_at_interval', 'bool', _('Autosave version on regular intervals'), False), # T: Label for plugin preference
+		('autosave_interval', 'int', _('Autosave interval in minutes'), 10, (1, 3600)), # T: Label for plugin preference
 	)
 
 	@classmethod
 	def check_dependencies(klass):
 		has_bzr = VCS.check_dependencies(VCS.BZR)
-		has_git  = VCS.check_dependencies(VCS.GIT)
-		has_hg  = VCS.check_dependencies(VCS.HG)
+		has_git = VCS.check_dependencies(VCS.GIT)
+		has_hg = VCS.check_dependencies(VCS.HG)
 		has_fossil = VCS.check_dependencies(VCS.FOSSIL)
 		#TODO parameterize the return, so that a new backend will be automatically available
-		return has_bzr|has_hg|has_git|has_fossil, [('bzr', has_bzr, False), ('hg', has_hg, False), ('git', has_git, False), ('fossil', has_fossil, False)]
-
-	def extend(self, obj):
-		name = obj.__class__.__name__
-		if name == 'MainWindow':
-			nb = obj.ui.notebook # XXX
-			nb_ext = self.get_extension(NotebookExtension, notebook=nb)
-			assert nb_ext, 'No notebook extension found for: %s' % nb
-			mw_ext = MainWindowExtension(self, obj, nb_ext)
-			self.extensions.add(mw_ext)
-		else:
-			PluginClass.extend(self, obj)
+		return has_bzr | has_hg | has_git | has_fossil, [('bzr', has_bzr, False), ('hg', has_hg, False), ('git', has_git, False), ('fossil', has_fossil, False)]
 
 
-@extends('Notebook')
-class NotebookExtension(ObjectExtension):
+class VersionControlNotebookExtension(NotebookExtension):
 
 	def __init__(self, plugin, notebook):
-		ObjectExtension.__init__(self, plugin, notebook)
-		self.plugin = plugin
-		self.notebook = notebook
+		NotebookExtension.__init__(self, plugin, notebook)
+		self.vcs = None
 		self.detect_vcs()
 
 	def _get_notebook_dir(self):
-		if self.notebook.dir:
-			return self.notebook.dir
-		elif self.notebook.file:
-			return self.notebook.file.dir
-		else:
-			assert False, 'Notebook is not based on a file or folder'
+		assert isinstance(self.notebook.folder, LocalFolder)
+		return self.notebook.folder
 
 	def detect_vcs(self):
-		dir = self._get_notebook_dir()
+		try:
+			dir = self._get_notebook_dir()
+		except AssertionError:
+			return
+
 		self.vcs = VCS.detect_in_folder(dir)
-		if self.vcs:
-			# HACK - FIXME use proper FS signals here
-			# git requires changes to be added to staging, bzr does not
-			# so add a hook for when page is written, to update staging.
-			#
-			# For a more generic behavior, the update_staging is implemented
-			# for all version control systems. If not required - eg. bzr, hg,
-			# then nothing is done
-			self.notebook.connect_after('stored-page', lambda o, n: self.vcs.update_staging() )
 
 	def init_vcs(self, vcs):
 		dir = self._get_notebook_dir()
 		self.vcs = VCS.create(vcs, dir, dir)
 
-		if self.vcs:
-			with self.notebook.lock:
-				self.vcs.init()
+		if self.vcs and not self.vcs.repo_exists():
+			with NotebookState(self.notebook):
+				self.vcs.init_repo()
+
 
 	def teardown(self):
 		if self.vcs:
 			self.vcs.disconnect_all()
 
 
-def _monitor_thread(thread):
-	if thread.done:
-		if thread.error:
-			error = thread.exc_info[1]
-			logger.error('Error during async commit', exc_info=thread.exc_info)
-			ErrorDialog(None, error, thread.exc_info).run() # XXX None should be window
-		return False # stop signal
-	else:
-		return True # keep handler
+class VersionControlMainWindowExtension(MainWindowExtension):
 
-def monitor_thread(thread):
-	gobject.idle_add(_monitor_thread, thread)
+	def __init__(self, plugin, window):
+		MainWindowExtension.__init__(self, plugin, window)
 
+		self.notebook_ext = find_extension(window.notebook, VersionControlNotebookExtension)
 
-@extends('MainWindow')
-class MainWindowExtension(WindowExtension):
-
-	uimanager_xml = '''
-	<ui>
-	<menubar name='menubar'>
-		<menu action='file_menu'>
-			<placeholder name='versioning_actions'>
-				<menuitem action='save_version'/>
-				<menuitem action='show_versions'/>
-			</placeholder>
-		</menu>
-	</menubar>
-	</ui>
-	'''
-
-	def __init__(self, plugin, window, notebook_ext):
-		WindowExtension.__init__(self, plugin, window)
-		self.notebook_ext = notebook_ext
 		self._autosave_thread = None
+		self._autosave_timer = None
 
 		if self.notebook_ext.vcs is None:
 			gaction = self.actiongroup.get_action('show_versions')
 			gaction.set_sensitive(False)
 		else:
-			if self.plugin.preferences['autosave']:
-				self.do_save_version_async()
+			self.on_preferences_changed(None, start=True)
 
-		def on_quit(o):
-			if self._autosave_thread and not self._autosave_thread.done:
-				self._autosave_thread.join()
-
-			if self.plugin.preferences['autosave']:
+		def on_close(o):
+			if self.plugin.preferences['autosave'] \
+			or self.plugin.preferences['autosave_at_interval']:
 				self.do_save_version()
 
-		self.window.ui.connect('quit', on_quit) # XXX
+		self.window.connect('close', on_close)
+
+		self.connectto(self.plugin.preferences, 'changed',
+			self.on_preferences_changed)
+
+	def on_preferences_changed(self, o, start=False):
+		self._stop_timer()
+
+		if (start and self.plugin.preferences['autosave']) \
+		or self.plugin.preferences['autosave_at_interval']:
+			self.do_save_version_async()
+
+		if self.plugin.preferences['autosave_at_interval']:
+			self._start_timer()
+
+	def _start_timer(self):
+		timeout = 60000 * self.plugin.preferences['autosave_interval']
+		self._autosave_timer = GObject.timeout_add(
+			timeout, self.do_save_version_async)
+
+	def _stop_timer(self):
+		if self._autosave_timer:
+			GObject.source_remove(self._autosave_timer)
+			self._autosave_timer = None
+
+	def teardown(self):
+		self._stop_timer()
 
 	def do_save_version_async(self, msg=None):
 		if not self.notebook_ext.vcs:
-			return
+			return False # stop timer
 
-		if self._autosave_thread and not self._autosave_thread.done:
-			self._autosave_thread.join()
+		if self._autosave_thread and self._autosave_thread.is_alive():
+			return True # continue time
 
-		self._autosave_thread = FunctionThread(self.do_save_version, (msg,))
-		self._autosave_thread.start()
-		monitor_thread(self._autosave_thread)
+		with NotebookState(self.notebook_ext.notebook):
+			op, thread = self._commit_op(msg)
+			self._autosave_thread = thread
+			op.run_on_idle()
+
+		return True # continue timer
+
+	def _commit_op(self, msg):
+		thread = threading.Thread(
+			target=partial(self._save_version, msg)
+		)
+		thread.start()
+		return SimpleAsyncOperation(
+			notebook=self.notebook_ext.notebook,
+			message='Saving version in progress',
+			thread=thread
+		), thread
 
 	def do_save_version(self, msg=None):
 		if not self.notebook_ext.vcs:
 			return
 
-		if not msg:
+		if self._autosave_thread and self._autosave_thread.is_alive():
+			self._autosave_thread.join()
+
+		with NotebookState(self.notebook_ext.notebook):
+			self._save_version(msg)
+
+	def _save_version(self, msg=None):
+		if msg is None:
 			msg = _('Automatically saved version from zim')
 				# T: default version comment for auto-saved versions
 
-		self.window.ui.assert_save_page_if_modified() # XXX
 		try:
-			self.notebook_ext.vcs.commit(msg)
+			self.notebook_ext.vcs.commit_version(msg)
 		except NoChangesError:
 			logger.debug('No autosave version needed - no changes')
 
-	@action(_('S_ave Version...'), 'gtk-save-as', '<Primary><shift>S', readonly=False) # T: menu item
+	@action(_('S_ave Version...'), '<Primary><shift>S', menuhints='notebook:edit') # T: menu item
 	def save_version(self):
-		self.window.ui.assert_save_page_if_modified() # XXX
-
 		if not self.notebook_ext.vcs:
 			vcs = VersionControlInitDialog(self.window).run()
+			logger.debug("Selected VCS: %s", vcs)
 			if vcs is None:
 				return # Canceled
 
@@ -213,18 +219,17 @@ class MainWindowExtension(WindowExtension):
 			if self.notebook_ext.vcs:
 				gaction = self.actiongroup.get_action('show_versions')
 				gaction.set_sensitive(True)
+				self.on_preferences_changed(None, start=False)
 
-		with self.notebook_ext.notebook.lock:
+		with NotebookState(self.notebook_ext.notebook):
 			SaveVersionDialog(self.window, self, self.notebook_ext.vcs).run()
 
-	@action(_('_Versions...')) # T: menu item
+	@action(_('_Versions...'), menuhints='notebook') # T: menu item
 	def show_versions(self):
-		self.window.ui.assert_save_page_if_modified() # XXX
-
 		dialog = VersionsDialog.unique(self, self.window,
 			self.notebook_ext.vcs,
 			self.notebook_ext.notebook,
-			self.window.ui.page # XXX
+			self.window.page
 		)
 		dialog.present()
 
@@ -251,7 +256,7 @@ class VCS(object):
 
 	# Enumeration of all available backends
 	BZR = _('Bazaar') # T: option value
-	HG  = _('Mercurial') # T: option value
+	HG = _('Mercurial') # T: option value
 	GIT = _('Git') # T: option value
 	FOSSIL = _('Fossil') # T: option value
 
@@ -260,7 +265,7 @@ class VCS(object):
 		"""Detect if a version control system has already been setup in the folder.
 		It also create the instance by calling the VCS.create() method
 		@param dir: a L{Dir} instance representing the notebook root folder
-		@returns: a L{VCSBackend} instance which will manage the versioning or C{None}
+		@returns: a vcs backend object or C{None}
 		"""
 		name, root = klass._detect_in_folder(dir)
 
@@ -284,31 +289,29 @@ class VCS(object):
 			return None
 
 	@classmethod
-	def _detect_in_folder(klass, dir):
-		# split off because it is easier to test this way
-		#
+	def _detect_in_folder(klass, folder):
 		# Included unsupported systems as well, to make sure we stop
 		# looking for parents if these are detected.
-		for path in reversed(list(dir)):
-			if path.subdir('.bzr').exists():
-				return 'bzr', path
-			elif path.subdir('.hg').exists():
-				return 'hg', path
-			elif path.subdir('.git').exists() or path.file('.git').exists():
-				return 'git', path
-			elif path.subdir('.svn').exists():
-				return 'svn', path
-			elif path.file('.fslckout').exists() or path.file('_FOSSIL_').exists():
-				return 'fossil', path
-			## Commented CVS out since it potentially
-			## conflicts with like-named pages
-			# elif path.subdir('CVS').exists():
-				# return 'cvs', path
-			##
-			else:
-				continue
+		if folder.folder('.bzr').exists():
+			return 'bzr', folder
+		elif folder.folder('.hg').exists():
+			return 'hg', folder
+		elif folder.folder('.git').exists() or folder.file('.git').exists():
+			return 'git', folder
+		elif folder.folder('.svn').exists():
+			return 'svn', folder
+		elif folder.file('.fslckout').exists() or folder.file('_FOSSIL_').exists():
+			return 'fossil', folder
+		## Commented CVS out since it potentially
+		## conflicts with like-named pages
+		# elif path.folder('CVS').exists():
+			# return 'cvs', path
+		##
 		else:
-			return None, None
+			try:
+				return klass._detect_in_folder(folder.parent()) # recurs
+			except ValueError:
+				return None, None
 
 	@classmethod
 	def get_backend(klass, vcs):
@@ -343,13 +346,13 @@ class VCS(object):
 		@param vcs_dir: a L{Dir} instance representing the VCS root folder
 		@param notebook_dir: a L{Dir} instance representing the notebook root folder
 		(must be equal to or below vcs_dir)
-		@returns: a C{VCSBackend} instance setup with the required backend
+		@returns: a vcs backend object
 		"""
 		if not (notebook_dir == vcs_dir or notebook_dir.ischild(vcs_dir)):
-			raise AssertionError, 'Notebook %s is not part of version control dir %s' % (notebook_dir, vcs_dir)
+			raise AssertionError('Notebook %s is not part of version control dir %s' % (notebook_dir, vcs_dir))
 
 		vcs_backend_klass = VCS.get_backend(vcs)
-		return VCSBackend(vcs_dir, vcs_backend_klass(vcs_dir, notebook_dir))
+		return vcs_backend_klass(vcs_dir, notebook_dir)
 
 	@classmethod
 	def check_dependencies(klass, vcs):
@@ -360,202 +363,9 @@ class VCS(object):
 		return VCS.get_backend(vcs).tryexec()
 
 
-
-class VCSBackend(ConnectorMixin):
-	"""Parent class for all VCS backend implementations.
-	It implements the required API.
-	"""
-
-	def __init__(self, dir, vcs_specific_app):
-		"""Initialize the instance in normal or test mode
-		- in case of TEST_MODE off, it checks the file system
-		  for creation, move or delete of files
-		- in case of TEST_MODE on, it does not check anything
-		  in order to avoid to interfer with dev environment
-
-		@param dir: a L{Dir} object representing the repository working directory path
-		@param vcs_specific_app: a backend object
-		"""
-		self._root = dir
-		self._lock = FS.get_async_lock(self._root)
-		self._app  = vcs_specific_app
-		if not TEST_MODE:
-			# Avoid touching the bazaar repository with zim sources
-			# when we write to tests/tmp etc.
-			self.connectto_all(FS, (
-				'path-created',
-				'path-moved',
-				'path-deleted'
-			) )
-
-	@property
-	def vcs(self):
-		return self._app
-
-	@property
-	def root(self):
-		return self._root
-
-	@property
-	def lock(self):
-		return self._lock
-
-
-	def _ignored(self, path):
-		"""Return True if we should ignore this path
-		TODO add specific ignore patterns in the _ignored_vcs_specific method
-		for now we just hardcode zim specific logic
-
-		@param path: a L{UnixFile} object representing the file path to check
-		@returns: True if the path should be ignored or False
-		"""
-		return '.zim' in path.split() or self.vcs._ignored(path)
-
-	def init(self):
-		"""Initialize a Bazaar repository in the self.root directory.
-		If the directory does not exist, then create it
-		@returns: nothing
-		"""
-		if self.vcs.repo_exists():
-			return
-
-		if not self.root.exists():
-			self.root.touch()
-
-		#~ with self.lock: # FIXME - conflicts with "git init" !???
-		self.vcs.init_repo()
-
-	def on_path_created(self, fs, path):
-		"""Callback to add a new file or folder when added to the wiki
-		Note: the VCS operation is asynchronous
-
-		@param fs: the L{FSSingletonClass} instance representing the file system
-		@param path: the L{UnixFile} object representing the newly created file or folder
-		@returns: nothing
-		"""
-		if path.ischild(self.root) and not self._ignored(path):
-			FunctionThread(self.vcs.add, (path,), lock=self._lock).start()
-
-
-	def on_path_moved(self, fs, oldpath, newpath):
-		"""Callback to move the file in Bazaar when moved in the wiki
-		Note: the VCS operation is asynchronous
-
-		@param fs: the L{FSSingletonClass} instance representing the file system
-		@param oldpath: the L{UnixFile} object representing the old path of the file or folder
-		@param newpath: the L{UnixFile} object representing the new path of the file or folder
-		@returns: nothing
-		"""
-		if newpath.ischild(self.root) and not self._ignored(newpath):
-			if oldpath.ischild(self.root):
-				# Parent of newpath needs to be versioned in order to make mv succeed
-				FunctionThread(self.vcs.move, (oldpath, newpath), lock=self._lock).start()
-			else:
-				FunctionThread(self.vcs.add, (newpath,), lock=self._lock).start()
-		elif oldpath.ischild(self.root) and not self._ignored(oldpath):
-			self.on_path_deleted(self, fs, oldpath)
-
-	def on_path_deleted(self, fs, path):
-		"""Callback to remove a file from Bazaar when deleted from the wiki
-		Note: the VCS operation is asynchronous
-
-		@param fs: the L{FSSingletonClass} instance representing the file system
-		@param path: the L{UnixFile} object representing the path of the file or folder to delete
-		@returns: nothing
-		"""
-		FunctionThread(self.vcs.remove, (path,), lock=self._lock).start()
-
-	@property
-	def modified(self):
-		"""return True if changes are detected, or False"""
-		return ''.join( self.get_status() ).strip() != ''
-		with self.lock:
-			return self.vcs.is_modified()
-
-	def get_status(self, **kwarg):
-		"""Returns repo status as a list of text lines
-
-		@returns: list of text lines (like a shell command result)
-		"""
-		status = list()
-		with self.lock:
-			status = self.vcs.status(**kwarg)
-		return status
-
-	def get_diff(self, versions=None, file=None):
-		"""Returns the diff operation result of a repo or file
-		@param versions: couple of version numbers (integer)
-		@param file: L{UnixFile} object of the file to check, or None
-		@returns: the diff result
-		"""
-		with self.lock:
-			nc = ['=== No Changes\n']
-			diff = self.vcs.diff(versions, file) or nc
-		return diff
-
-	def get_annotated(self, file, version=None):
-		"""Returns the annotated version of a file
-		@param file: L{UnixFile} object of the file to check, or None
-		@param version: required version number (integer) or None
-		@returns: the annotated version of the file result
-		"""
-		with self.lock:
-			annotated = self.vcs.annotate(file, version)
-		return annotated
-
-	def commit(self, msg):
-		"""Run a commit operation.
-
-		@param msg: commit message (str)
-		@returns: nothing
-		"""
-		with self.lock:
-			self._commit(msg)
-
-	def _commit(self, msg):
-		stat = ''.join(self.vcs.status()).strip()
-		if not stat:
-			raise NoChangesError(self.root)
-		else:
-			self.vcs.add()
-			self.vcs.commit(None, msg)
-
-	def revert(self, version=None, file=None):
-		with self.lock:
-			self.vcs.revert(file, version)
-
-
-	def list_versions(self, file=None):
-		"""Returns a list of all versions, for a file or for the entire repo
-
-		@param file: a L{UnixFile} object representing the path to the file, or None
-		@returns: a list of tuples (revision (int), date, user (str), msg (str))
-		"""
-		# TODO see if we can get this directly from bzrlib as well
-		with self.lock:
-			lines = self.vcs.log(file)
-			versions = self.vcs.log_to_revision_list(lines)
-		return versions
-
-
-	def get_version(self, file, version):
-		"""FIXME Document"""
-		with self.lock:
-			version = self.vcs.cat(file, version)
-		return version
-
-	def update_staging(self):
-		with self.lock:
-			self.vcs.stage()
-
-
-class VCSApplicationBase(object):
+class VCSApplicationBase(ConnectorMixin):
 	"""This class is the base class for the classes representing the
 	specific version control applications.
-
-	This class is abstract and must be inherited. Subclasses of this
-	class can be used by L{VCSBackend} to apply version control to
-	a folder.
 	"""
 
 	def __init__(self, vcs_dir, notebook_dir):
@@ -564,12 +374,54 @@ class VCSApplicationBase(object):
 		@param notebook_dir: a L{Dir} instance representing the notebook root folder
 		(must be equal to or below vcs_dir)
 		"""
+		assert isinstance(vcs_dir, LocalFolder)
+
 		if not (notebook_dir == vcs_dir or notebook_dir.ischild(vcs_dir)):
-			raise AssertionError, 'Notebook %s is not part of version control dir %s' % (notebook_dir, vcs_dir)
+			raise AssertionError('Notebook %s is not part of version control dir %s' % (notebook_dir, vcs_dir))
 		self._app = self.build_bin_application_instance()
 		self.root = vcs_dir
 		self.notebook_dir = notebook_dir
 
+		if notebook_dir.watcher is None:
+			from zim.newfs.helpers import FileTreeWatcher
+			notebook_dir.watcher = FileTreeWatcher()
+
+		self.connectto_all(
+			notebook_dir.watcher,
+			('created', 'moved', 'removed')
+		)
+
+	def on_created(self, fs, path):
+		"""Callback when a file has been created
+		@param fs: the watcher object
+		@param path: a L{File} or L{Folder} object
+		"""
+		if path.ischild(self.root) and not self._ignored(path):
+			self.add(path)
+
+	def on_moved(self, fs, oldpath, newpath):
+		"""Callback when a file has been moved
+		@param fs: the watcher object
+		@param oldpath: a L{File} or L{Folder} object
+		@param newpath: a L{File} or L{Folder} object
+		"""
+		if newpath.ischild(self.root) and not self._ignored(newpath):
+			if oldpath.ischild(self.root):
+				# Parent of newpath needs to be versioned in order to make mv succeed
+				self.move(oldpath, newpath)
+			else:
+				self.add(newpath)
+		elif oldpath.ischild(self.root) and not self._ignored(oldpath):
+			self.on_path_deleted(self, fs, oldpath)
+
+	def on_removed(self, fs, path):
+		"""Callback when a file has been delted
+		@param fs: the watcher object
+		@param path: a L{File} or L{Folder} object
+		"""
+		if path.ischild(self.root) and not self._ignored(path):
+			self.remove(path)
+
 	@classmethod
 	def build_bin_application_instance(cls):
 		"""Builds an L{Application} object for the backend command
@@ -611,7 +463,7 @@ class VCSApplicationBase(object):
 		@implementation: may be overridden if some files are to be ignored \
 		                 specifically for the backend
 		"""
-		return False
+		return '.zim' in file.pathnames
 
 	########
 	#
@@ -630,7 +482,7 @@ class VCSApplicationBase(object):
 		"""
 		raise NotImplementedError
 
-	def annotate(self, file, version):
+	def annotate(self, file, version=None):
 		"""return the annotated version of a file. This is commonly related
 		to the VCS command annotate
 
@@ -655,6 +507,18 @@ class VCSApplicationBase(object):
 		"""
 		raise NotImplementedError
 
+	def commit_version(self, msg):
+		"""Run a commit operation.
+
+		@param msg: commit message (str)
+		@returns: nothing
+		"""
+		if self.is_modified():
+			self.add()
+			self.commit(None, msg)
+		else:
+			raise NoChangesError(self.root)
+
 	def commit(self, file, msg):
 		"""Execute a commit for the file or for the entire repository
 		@param file: a L{File} instance representing the file or None for the entire repository
@@ -671,7 +535,7 @@ class VCSApplicationBase(object):
 		"""
 		raise NotImplementedError
 
-	def diff(self, versions, file=None):
+	def diff(self, versions=None, file=None):
 		"""Returns the result of a diff between two revisions as a list of str()
 		representing the diff operation output.
 		@param versions: int, str, couple or tuple representing the versions to compare
@@ -742,6 +606,17 @@ class VCSApplicationBase(object):
 		"""
 		raise NotImplementedError
 
+	def list_versions(self, file=None):
+		"""Returns a list of all versions, for a file or for the entire repo
+
+		@param file: a L{File} object representing the path to the file, or None
+		@returns: a list of tuples (revision (int), date, user (str), msg (str))
+		"""
+		# TODO see if we can get this directly from bzrlib as well
+		lines = self.log(file)
+		versions = self.log_to_revision_list(lines)
+		return versions
+
 	def log(self, file=None):
 		"""Returns the history related to a file.
 		@param file: a L{File} instance representing the file or None (for the entire repository)
@@ -788,7 +663,7 @@ class VCSApplicationBase(object):
 
 	def remove(self, file):
 		"""Remove a file from the repository.
-		@param file: a L{File} instance representing the file that have been deleted from the FS
+		@param file: a L{File} instance representing the file that have been deleted
 		@returns: C{True} if the command was successfull
 		@implementation: must be implemented in child class. \
 		                 CAUTION: this must implement the VCS operation required \
@@ -799,7 +674,7 @@ class VCSApplicationBase(object):
 		"""
 		raise NotImplementedError
 
-	def revert(self, file, version):
+	def revert(self, file=None, version=None):
 		"""Reverts a file to an older version
 		@param file: a L{File} instance representing the file or None for the entire repo
 		@param version: a str() or int() representing the expected version
@@ -813,10 +688,7 @@ class VCSApplicationBase(object):
 		raise NotImplementedError
 
 	def stage(self):
-		"""Fixme - to be documented
-		Usefull for git, runs:
-		  git add -u
-		  git add -A
+		"""Prepares the repo for a commit. Used, for example, by git to stage changes so that the status message in SaveVersionDialog shows what will be committed.
 
 		@implementation: optional to be implemented in child class
 		"""
@@ -843,84 +715,89 @@ def get_side_by_side_app():
 		return None
 
 
-class VersionControlInitDialog(QuestionDialog):
+class VersionControlInitDialog(Dialog):
 
-	def __init__(self, ui):
-		QuestionDialog.__init__(self, ui, (
-			_("Enable Version Control?"), # T: Question dialog
+	def __init__(self, parent):
+		Dialog.__init__(self, parent, _("Enable Version Control?")) # T: Question dialog
+		self.add_text(
 			_("Version control is currently not enabled for this notebook.\n"
-			  "Do you want to enable it?" ) # T: Detailed question
-		) )
+			  "Do you want to enable it?") # T: Detailed question
+		)
 
-		self.combobox = gtk.combo_box_new_text()
+		self.combobox = Gtk.ComboBoxText()
 		for option in (VCS.BZR, VCS.GIT, VCS.HG, VCS.FOSSIL):
 			if VCS.check_dependencies(option):
 				self.combobox.append_text(option)
 		self.combobox.set_active(0)
 
-		hbox = gtk.HBox(spacing=5)
-		hbox.pack_end(self.combobox, False)
-		hbox.pack_end(gtk.Label(_('Backend') + ':'), False)
+		hbox = Gtk.Box(spacing=5)
+		hbox.add(Gtk.Label(_('Backend') + ':'))
+		hbox.add(self.combobox)
 			# T: option to chose versioncontrol backend
-		self.vbox.pack_start(hbox, False)
+		hbox.set_halign(Gtk.Align.CENTER)
+		self.vbox.pack_start(hbox, False, False, 0)
 		hbox.show_all()
 
-	def run(self):
-		if QuestionDialog.run(self):
-			return self.combobox.get_active_text()
-		else:
-			return None
+	def do_response_ok(self):
+		self.result = self.combobox.get_active_text()
+		return True
 
 
 class SaveVersionDialog(Dialog):
 
-	def __init__(self, ui, window_ext, vcs):
-		Dialog.__init__(self, ui, _('Save Version'), # T: dialog title
-			button=(None, 'gtk-save'), help='Plugins:Version Control')
+	def __init__(self, parent, window_ext, vcs):
+		Dialog.__init__(
+			self,
+			parent,
+			_('Save Version'), # T: dialog title
+			button=_('_Save'), # T: button label
+			help='Plugins:Version Control'
+		)
 		self.window_ext = window_ext
 		self.vcs = vcs
 
-		self.vbox.pack_start(
-			gtk.Label(_("Please enter a comment for this version")), False)  # T: Dialog text
+		label = Gtk.Label(_("Please enter a comment for this version"))  # T: Dialog text
+		self.vbox.pack_start(label, False, True, 0)
 
 		vpaned = VPaned()
-		self.vbox.add(vpaned)
+		self.vbox.pack_start(vpaned, True, True, 0)
 
 		window, self.textview = ScrolledTextView(_('Saved version from zim'))
 			# T: default version comment in the "save version" dialog
 		self.textview.set_editable(True)
 		vpaned.add1(window)
 
-		vbox = gtk.VBox()
+		vbox = Gtk.VBox()
 		vpaned.add2(vbox)
 
-		label = gtk.Label('<b>'+_('Details')+'</b>')
+		label = Gtk.Label(label='<b>' + _('Details') + '</b>')
 			# T: section for version details in "save version" dialog
 		label.set_use_markup(True)
 		label.set_alignment(0, 0.5)
-		vbox.pack_start(label, False)
+		vbox.pack_start(label, False, True, 0)
 
-		status = self.vcs.get_status()
+		self.vcs.stage()
+		status = self.vcs.status()
 		window, textview = ScrolledTextView(text=''.join(status), monospace=True)
 		vbox.add(window)
 
 	def do_response_ok(self):
-		# notebook.lock already set by plugin.save_version()
 		buffer = self.textview.get_buffer()
 		start, end = buffer.get_bounds()
-		msg = buffer.get_text(start, end, False).strip()
-		if msg:
-			self.window_ext.do_save_version_async(msg)
-			return True
-		else:
+		msg = start.get_text(end).strip()
+		if not msg:
 			return False
 
+		op, thread = self.window_ext._commit_op(msg)
+		ProgressDialog(self, op).run()
+		return True
+
 
 class VersionsDialog(Dialog):
 
-	def __init__(self, ui, vcs, notebook, page=None):
-		Dialog.__init__(self, ui, _('Versions'), # T: dialog title
-			buttons=gtk.BUTTONS_CLOSE, help='Plugins:Version Control')
+	def __init__(self, parent, vcs, notebook, page=None):
+		Dialog.__init__(self, parent, _('Versions'), # T: dialog title
+			buttons=Gtk.ButtonsType.CLOSE, help='Plugins:Version Control')
 		self.notebook = notebook
 		self.vcs = vcs
 		self._side_by_side_app = get_side_by_side_app()
@@ -930,46 +807,46 @@ class VersionsDialog(Dialog):
 
 		self.vpaned = VPaned()
 		self.vpaned.set_position(self.uistate['vpanepos'])
-		self.vbox.add(self.vpaned)
+		self.vbox.pack_start(self.vpaned, True, True, 0)
 
-		vbox = gtk.VBox(spacing=5)
+		vbox = Gtk.VBox(spacing=5)
 		self.vpaned.pack1(vbox, resize=True)
 
 		# Choice between whole notebook or page
-		label = gtk.Label('<b>'+_('Versions')+':</b>') # section label
+		label = Gtk.Label(label='<b>' + _('Versions') + ':</b>') # section label
 		label.set_use_markup(True)
 		label.set_alignment(0, 0.5)
-		vbox.pack_start(label, False)
+		vbox.pack_start(label, False, True, 0)
 
-		self.notebook_radio = gtk.RadioButton(None, _('Complete _notebook'))
+		self.notebook_radio = Gtk.RadioButton.new_with_mnemonic_from_widget(None, _('Complete _notebook'))
 			# T: Option in versions dialog to show version for complete notebook
-		self.page_radio = gtk.RadioButton(self.notebook_radio, _('_Page')+':')
+		self.page_radio = Gtk.RadioButton.new_with_mnemonic_from_widget(self.notebook_radio, _('_Page') + ':')
 			# T: Option in versions dialog to show version for single page
-		#~ recursive_box = gtk.CheckButton('Recursive')
-		vbox.pack_start(self.notebook_radio, False)
+		#~ recursive_box = Gtk.CheckButton.new_with_mnemonic('Recursive')
+		vbox.pack_start(self.notebook_radio, False, True, 0)
 
 		# Page entry
-		hbox = gtk.HBox(spacing=5)
-		vbox.pack_start(hbox, False)
-		hbox.pack_start(self.page_radio, False)
+		hbox = Gtk.HBox(spacing=5)
+		vbox.pack_start(hbox, False, True, 0)
+		hbox.pack_start(self.page_radio, False, True, 0)
 		self.page_entry = PageEntry(self.notebook)
 		if page:
 			self.page_entry.set_path(page)
-		hbox.pack_start(self.page_entry, False)
+		hbox.pack_start(self.page_entry, False, True, 0)
 
 		# View annotated button
-		ann_button = gtk.Button(_('View _Annotated')) # T: Button label
+		ann_button = Gtk.Button.new_with_mnemonic(_('View _Annotated')) # T: Button label
 		ann_button.connect('clicked', lambda o: self.show_annotated())
-		hbox.pack_start(ann_button, False)
+		hbox.pack_start(ann_button, False, True, 0)
 
 		# Help text
-		label = gtk.Label('<i>\n'+_( '''\
+		label = Gtk.Label(label='<i>\n' + _( '''\
 Select a version to see changes between that version and the current
 state. Or select multiple versions to see changes between those versions.
-''' ).strip()+'</i>') # T: Help text in versions dialog
+''' ).strip() + '</i>') # T: Help text in versions dialog
 		label.set_use_markup(True)
 		#~ label.set_alignment(0, 0.5)
-		vbox.pack_start(label, False)
+		vbox.pack_start(label, False, True, 0)
 
 		# Version list
 		self.versionlist = VersionsTreeView()
@@ -978,43 +855,43 @@ state. Or select multiple versions to see changes between those versions.
 		vbox.add(scrolled)
 
 		col = self.uistate.setdefault('sortcol', self.versionlist.REV_SORT_COL)
-		order = self.uistate.setdefault('sortorder', gtk.SORT_DESCENDING)
+		order = self.uistate.setdefault('sortorder', Gtk.SortType.DESCENDING)
 		try:
 			self.versionlist.get_model().set_sort_column_id(col, order)
 		except:
 			logger.exception('Invalid sort column: %s %s', col, order)
 
 		# -----
-		vbox = gtk.VBox(spacing=5)
+		vbox = Gtk.VBox(spacing=5)
 		self.vpaned.pack2(vbox, resize=False)
 
-		label = gtk.Label('<b>'+_('Comment')+'</b>') # T: version details
+		label = Gtk.Label(label='<b>' + _('Comment') + '</b>') # T: version details
 		label.set_use_markup(True)
 		label.set_alignment(0.0, 0.5)
-		vbox.pack_start(label, False)
+		vbox.pack_start(label, False, True, 0)
 
 		# Comment text
 		window, textview = ScrolledTextView()
 		self.comment_textview = textview
 		vbox.add(window)
 
-		buttonbox = gtk.HButtonBox()
-		buttonbox.set_layout(gtk.BUTTONBOX_END)
-		vbox.pack_start(buttonbox, False)
+		buttonbox = Gtk.HButtonBox()
+		buttonbox.set_layout(Gtk.ButtonBoxStyle.END)
+		vbox.pack_start(buttonbox, False, True, 0)
 
 		# Restore version button
-		revert_button = gtk.Button(_('_Restore Version')) # T: Button label
+		revert_button = Gtk.Button.new_with_mnemonic(_('_Restore Version')) # T: Button label
 		revert_button.connect('clicked', lambda o: self.restore_version())
 		buttonbox.add(revert_button)
 
 		# Notebook Changes button
-		diff_button = gtk.Button(_('Show _Changes'))
+		diff_button = Gtk.Button.new_with_mnemonic(_('Show _Changes'))
 			# T: button in versions dialog for diff
 		diff_button.connect('clicked', lambda o: self.show_changes())
 		buttonbox.add(diff_button)
 
 		# Compare page button
-		comp_button = gtk.Button(_('_Side by Side'))
+		comp_button = Gtk.Button.new_with_mnemonic(_('_Side by Side'))
 			# T: button in versions dialog for side by side comparison
 		comp_button.connect('clicked', lambda o: self.show_side_by_side())
 		buttonbox.add(comp_button)
@@ -1071,7 +948,7 @@ state. Or select multiple versions to see changes between those versions.
 		# select last version
 		self.versionlist.get_selection().select_path((0,))
 		col = self.versionlist.get_column(0)
-		self.versionlist.row_activated(0, col)
+		self.versionlist.row_activated(Gtk.TreePath((0,)), col)
 
 	def save_uistate(self):
 		self.uistate['vpanepos'] = self.vpaned.get_position()
@@ -1091,9 +968,8 @@ state. Or select multiple versions to see changes between those versions.
 				return None # TODO error message valid page name?
 
 			if page \
-			and hasattr(page, 'source') \
-			and isinstance(page.source, File) \
-			and page.source.ischild(self.vcs.root):
+			and page.source_file is not None \
+			and page.source_file.ischild(self.vcs.root):
 				return page.source
 			else:
 				return None # TODO error message ?
@@ -1102,7 +978,7 @@ state. Or select multiple versions to see changes between those versions.
 		# TODO check for gannotated
 		file = self._get_file()
 		assert not file is None
-		annotated = self.vcs.get_annotated(file)
+		annotated = self.vcs.annotated(file)
 		TextDialog(self, _('Annotated Page Source'), annotated).run()
 			# T: dialog title
 
@@ -1118,16 +994,16 @@ state. Or select multiple versions to see changes between those versions.
 			  'All changes since the last saved version will be lost !')
 			  % {'page': path.name, 'version': str(version)}
 			  # T: Detailed question, "%(page)s" is replaced by the page, "%(version)s" by the version id
-		) ).run():
-			self.vcs.revert(file=file, version=version)
-			self.ui.reload_page() # XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
-			# TODO trigger vcs autosave here?
+		)).run():
+			self.vcs.revert(file, version)
+			page = self.notebook.get_page(path)
+			page.check_source_changed()
 
 	def show_changes(self):
 		# TODO check for gdiff
 		file = self._get_file()
 		versions = self.versionlist.get_versions()
-		diff = self.vcs.get_diff(file=file, versions=versions)
+		diff = self.vcs.diff(file=file, versions=versions) or ['=== No Changes\n']
 		TextDialog(self, _('Changes'), diff).run()
 			# T: dialog title
 
@@ -1137,7 +1013,7 @@ state. Or select multiple versions to see changes between those versions.
 		if not (file and versions):
 			raise AssertionError
 
-		files = map(lambda v: self._get_tmp_file(file, v), versions)
+		files = [self._get_tmp_file(file, v) for v in versions]
 		if len(files) == 1:
 			tmp = TmpFile(file.basename + '--CURRENT', persistent=True)
 				# need to be persistent, else it is cleaned up before application spawned
@@ -1147,7 +1023,7 @@ state. Or select multiple versions to see changes between those versions.
 		self._side_by_side_app.spawn(files)
 
 	def _get_tmp_file(self, file, version):
-		text = self.vcs.get_version(file, version)
+		text = self.vcs.cat(file, version)
 		tmp = TmpFile(file.basename + '--REV%s' % version, persistent=True)
 			# need to be persistent, else it is cleaned up before application spawned
 		tmp.writelines(text)
@@ -1156,12 +1032,12 @@ state. Or select multiple versions to see changes between those versions.
 
 class TextDialog(Dialog):
 
-	def __init__(self, ui, title, lines):
-		Dialog.__init__(self, ui, title, buttons=gtk.BUTTONS_CLOSE)
+	def __init__(self, parent, title, lines):
+		Dialog.__init__(self, parent, title, buttons=Gtk.ButtonsType.CLOSE)
 		self.set_default_size(600, 300)
 		self.uistate.setdefault('windowsize', (600, 500), check=value_is_coord)
 		window, textview = ScrolledTextView(''.join(lines), monospace=True)
-		self.vbox.add(window)
+		self.vbox.pack_start(window, True, True, 0)
 
 
 class VersionsTreeView(SingleClickTreeView):
@@ -1176,20 +1052,21 @@ class VersionsTreeView(SingleClickTreeView):
 	MSG_COL = 4
 
 	def __init__(self):
-		model = gtk.ListStore(str, str, str, str, str)
+		model = Gtk.ListStore(str, str, str, str, str)
 			# REV_SORT_COL, REV_COL, DATE_COL, USER_COL, MSG_COL
-		gtk.TreeView.__init__(self, model)
+		GObject.GObject.__init__(self)
+		self.set_model(model)
 
-		self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+		self.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
 		self.set_rubber_banding(True)
 
-		cell_renderer = gtk.CellRendererText()
+		cell_renderer = Gtk.CellRendererText()
 		for name, i in (
 			(_('Rev'), self.REV_COL), # T: Column header versions dialog
 			(_('Date'), self.DATE_COL), # T: Column header versions dialog
 			(_('Author'), self.USER_COL), # T: Column header versions dialog
 		):
-			column = gtk.TreeViewColumn(name, cell_renderer, text=i)
+			column = Gtk.TreeViewColumn(name, cell_renderer, text=i)
 			if i == self.REV_COL:
 				column.set_sort_column_id(self.REV_SORT_COL)
 			else:
@@ -1200,13 +1077,13 @@ class VersionsTreeView(SingleClickTreeView):
 
 			self.append_column(column)
 
-		model.set_sort_column_id(self.REV_SORT_COL, gtk.SORT_DESCENDING)
+		model.set_sort_column_id(self.REV_SORT_COL, Gtk.SortType.DESCENDING)
 			# By default sort by rev
 
 	def load_versions(self, versions):
 		model = self.get_model()
 		model.clear() # Empty for when we update
-		model.set_sort_column_id(self.REV_SORT_COL, gtk.SORT_DESCENDING)
+		model.set_sort_column_id(self.REV_SORT_COL, Gtk.SortType.DESCENDING)
 			# By default sort by rev
 
 		for version in versions:
