@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2008-2014 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
@@ -51,7 +50,7 @@ import logging
 
 logger = logging.getLogger('zim.templates')
 
-from zim.fs import File, Dir, PathLookupError
+from zim.fs import File, Dir, PathLookupError, FileNotFoundError, adapt_from_newfs
 from zim.config import data_dirs
 from zim.parsing import is_path_re
 from zim.signals import SignalEmitter
@@ -100,7 +99,7 @@ def get_template(category, template):
 	the template if a template name is given
 	@param template: the template name or file path
 	'''
-	assert isinstance(template, basestring)
+	assert isinstance(template, str)
 
 	if is_path_re.match(template):
 		file = File(template)
@@ -118,11 +117,11 @@ def get_template(category, template):
 		else:
 			file = File(template)
 			if not file.exists():
-				raise PathLookupError, _('Could not find template "%s"') % template
+				raise PathLookupError(_('Could not find template "%s"') % template)
 					# T: Error message in template lookup
 
 	if not file.exists():
-		raise PathLookupError, _('No such file: %s') % file
+		raise PathLookupError(_('No such file: %s') % file)
 			# T: Error message in template lookup
 
 	logger.info('Loading template from: %s', file)
@@ -135,6 +134,8 @@ class Template(SignalEmitter):
 	'''This class defines the main interface for templates
 	It takes care of parsing a template file and allows evaluating
 	the template with a given set of template parameters.
+
+	@signal: C{process (output, context)}: emitted by the "process" method
 	'''
 
 	# On purpose a very thin class, allow to test all steps of parsing
@@ -142,24 +143,32 @@ class Template(SignalEmitter):
 
 	# For templates that we define inline, use a file-like text buffer
 
+	__signals__ = {
+		'process': (None, None, (object, object))
+	}
+
 	template_functions = build_template_functions()
 
 	def __init__(self, file):
 		'''Constructor
 		@param file: a L{File} object for the template file
 		'''
+		file = adapt_from_newfs(file)
 		self.filename = file.path
 		try:
 			self.parts = TemplateParser().parse(file.read())
-		except Exception, error:
+		except Exception as error:
 			error.parser_file = file
 			raise
 
-		rdir = file.dir.subdir(file.basename[:-5]) # XXX strip extension, .html here
-		if rdir.exists():
-			self.resources_dir = rdir
-		else:
-			self.resources_dir = None
+		self.resources_dir = None
+		if '.' in file.basename:
+			name, ext = file.basename.rsplit('.')
+			rdir = file.dir.subdir(name)
+			if rdir.exists():
+				self.resources_dir = rdir
+
+		self._resources_cache = {}
 
 	def process(self, output, context):
 		'''Evaluate the template
@@ -168,12 +177,29 @@ class Template(SignalEmitter):
 		@param context: a C{dict} with a set of template parameters.
 		This dict is copied to prevent changes to the original dict when
 		processing the template
+		@emits: process
 		'''
 		context = TemplateContextDict(dict(context)) # COPY to keep changes local
 		context.update(self.template_functions) # set builtins
 		self.emit('process', output, context)
 
 	def do_process(self, output, context):
-		processor = TemplateProcessor(self.parts)
+		if self.resources_dir:
+			processor = TemplateProcessor(self.parts, self.parse_included_file)
+		else:
+			processor = TemplateProcessor(self.parts)
 		processor.process(output, context)
 
+	def parse_included_file(self, path):
+		if path not in self._resources_cache:
+			file = self.resources_dir.file(path)
+			if not file.exists():
+				raise FileNotFoundError(file)
+
+			try:
+				self._resources_cache[path] = TemplateParser().parse(file.read())
+			except Exception as error:
+				error.parser_file = file
+				raise
+
+		return self._resources_cache[path]
